#include "pluginInterface.h"

#include "elements.h"


RobotsMetamodelPlugin::RobotsMetamodelPlugin()
{
	initPlugin();
}

void RobotsMetamodelPlugin::initPlugin()
{
	initNameMap();
	initNodesAndEdgesSets();
	initMouseGestureMap();
	initPropertyMap();
	initPropertyDefaultsMap();
	initDescriptionMap();
	initPortTypes();
	initParentsMap();
	initPaletteGroupsMap();
	initPaletteGroupsDescriptionMap();
	initShallPaletteBeSortedMap();
	initExplosionsMap();
}

void RobotsMetamodelPlugin::initNameMap()
{
	mDiagramNameMap["RobotsDiagram"] = tr("Robot`s Behaviour Diagram");
	mDiagramNodeNameMap["RobotsDiagram"] = "RobotsDiagramGroup";

	mElementsNameMap["RobotsDiagram"]["ClearScreen"] = tr("Clear Screen");
	mElementsNameMap["RobotsDiagram"]["CommentBlock"] = tr("Comment");
	mElementsNameMap["RobotsDiagram"]["ControlFlow"] = tr("Link");
	mElementsNameMap["RobotsDiagram"]["Ev3Beep"] = tr("Beep");
	mElementsNameMap["RobotsDiagram"]["Ev3ClearEncoder"] = tr("Clear Encoder");
	mElementsNameMap["RobotsDiagram"]["Ev3DrawCircle"] = tr("Draw Circle");
	mElementsNameMap["RobotsDiagram"]["Ev3DrawLine"] = tr("Draw Line");
	mElementsNameMap["RobotsDiagram"]["Ev3DrawPixel"] = tr("Draw Pixel");
	mElementsNameMap["RobotsDiagram"]["Ev3DrawRect"] = tr("Draw Rectangle");
	mElementsNameMap["RobotsDiagram"]["Ev3EnginesBackward"] = tr("Motors Backward");
	mElementsNameMap["RobotsDiagram"]["Ev3EnginesForward"] = tr("Motors Forward");
	mElementsNameMap["RobotsDiagram"]["Ev3EnginesStop"] = tr("Stop Motors");
	mElementsNameMap["RobotsDiagram"]["Ev3Led"] = tr("Led");
	mElementsNameMap["RobotsDiagram"]["Ev3PlayTone"] = tr("Play Tone");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForButton"] = tr("Wait for button");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForColor"] = tr("Wait for Color");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForColorIntensity"] = tr("Wait for Color Intensity");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForEncoder"] = tr("Wait for Encoder");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForLight"] = tr("Wait for Light");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForSonarDistance"] = tr("Wait for Range Sensor");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForSound"] = tr("Wait for Sound Sensor");
	mElementsNameMap["RobotsDiagram"]["Ev3WaitForTouchSensor"] = tr("Wait for Touch Sensor");
	mElementsNameMap["RobotsDiagram"]["FiBlock"] = tr("End if");
	mElementsNameMap["RobotsDiagram"]["FinalNode"] = tr("Final Node");
	mElementsNameMap["RobotsDiagram"]["Fork"] = tr("Fork");
	mElementsNameMap["RobotsDiagram"]["Function"] = tr("Function");
	mElementsNameMap["RobotsDiagram"]["GetButtonCode"] = tr("Get Button Code");
	mElementsNameMap["RobotsDiagram"]["IfBlock"] = tr("Condition");
	mElementsNameMap["RobotsDiagram"]["InitialNode"] = tr("Initial Node");
	mElementsNameMap["RobotsDiagram"]["Join"] = tr("Join");
	mElementsNameMap["RobotsDiagram"]["KillThread"] = tr("Kill Thread");
	mElementsNameMap["RobotsDiagram"]["Loop"] = tr("Loop");
	mElementsNameMap["RobotsDiagram"]["MarkerDown"] = tr("Marker Down");
	mElementsNameMap["RobotsDiagram"]["MarkerUp"] = tr("Marker Up");
	mElementsNameMap["RobotsDiagram"]["NxtBeep"] = tr("Beep");
	mElementsNameMap["RobotsDiagram"]["NxtClearEncoder"] = tr("Clear Encoder");
	mElementsNameMap["RobotsDiagram"]["NxtDrawCircle"] = tr("Draw Circle");
	mElementsNameMap["RobotsDiagram"]["NxtDrawLine"] = tr("Draw Line");
	mElementsNameMap["RobotsDiagram"]["NxtDrawPixel"] = tr("Draw Pixel");
	mElementsNameMap["RobotsDiagram"]["NxtDrawRect"] = tr("Draw Rectangle");
	mElementsNameMap["RobotsDiagram"]["NxtEnginesBackward"] = tr("Motors Backward");
	mElementsNameMap["RobotsDiagram"]["NxtEnginesForward"] = tr("Motors Forward");
	mElementsNameMap["RobotsDiagram"]["NxtEnginesStop"] = tr("Stop Motors");
	mElementsNameMap["RobotsDiagram"]["NxtPlayTone"] = tr("Play Tone");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForButton"] = tr("Wait for Button");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForColor"] = tr("Wait for Color");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForColorIntensity"] = tr("Wait for Color Intensity");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForEncoder"] = tr("Wait for Encoder");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForLight"] = tr("Wait for Light");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForSonarDistance"] = tr("Wait for Sonar Distance");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForSound"] = tr("Wait for Sound Sensor");
	mElementsNameMap["RobotsDiagram"]["NxtWaitForTouchSensor"] = tr("Wait for Touch Sensor");
	mElementsNameMap["RobotsDiagram"]["PrintText"] = tr("Print Text");
	mElementsNameMap["RobotsDiagram"]["Randomizer"] = tr("Random Initialization");
	mElementsNameMap["RobotsDiagram"]["ReceiveMessageThreads"] = tr("Receive Message From Thread");
	mElementsNameMap["RobotsDiagram"]["RobotsDiagramNode"] = tr("Robot`s Behaviour Diagram");
	mElementsNameMap["RobotsDiagram"]["SendMessageThreads"] = tr("Send Message To Thread");
	mElementsNameMap["RobotsDiagram"]["Subprogram"] = tr("Subprogram");
	mElementsNameMap["RobotsDiagram"]["SubprogramDiagram"] = tr("Subprogram Diagram");
	mElementsNameMap["RobotsDiagram"]["SwitchBlock"] = tr("Switch");
	mElementsNameMap["RobotsDiagram"]["Timer"] = tr("Timer");
	mElementsNameMap["RobotsDiagram"]["TrikAngularServo"] = tr("Angular Servo");
	mElementsNameMap["RobotsDiagram"]["TrikDetect"] = tr("Detect by Videocamera");
	mElementsNameMap["RobotsDiagram"]["TrikDetectorToVariable"] = tr("Line Detector into Variable");
	mElementsNameMap["RobotsDiagram"]["TrikDrawArc"] = tr("Draw Arc");
	mElementsNameMap["RobotsDiagram"]["TrikDrawEllipse"] = tr("Draw Ellipse");
	mElementsNameMap["RobotsDiagram"]["TrikDrawLine"] = tr("Draw Line");
	mElementsNameMap["RobotsDiagram"]["TrikDrawPixel"] = tr("Draw Pixel");
	mElementsNameMap["RobotsDiagram"]["TrikDrawRect"] = tr("Draw Rectangle");
	mElementsNameMap["RobotsDiagram"]["TrikInitCamera"] = tr("Initialize Videocamera");
	mElementsNameMap["RobotsDiagram"]["TrikInitVideoStreaming"] = tr("Enable Video Streaming");
	mElementsNameMap["RobotsDiagram"]["TrikLed"] = tr("Led");
	mElementsNameMap["RobotsDiagram"]["TrikPlayTone"] = tr("Play Tone");
	mElementsNameMap["RobotsDiagram"]["TrikRemoveFile"] = tr("Remove File");
	mElementsNameMap["RobotsDiagram"]["TrikSadSmile"] = tr("Sad Smile");
	mElementsNameMap["RobotsDiagram"]["TrikSay"] = tr("Say");
	mElementsNameMap["RobotsDiagram"]["TrikSendMessage"] = tr("Send message");
	mElementsNameMap["RobotsDiagram"]["TrikSetBackground"] = tr("Background Color");
	mElementsNameMap["RobotsDiagram"]["TrikSetPainterColor"] = tr("Painter Color");
	mElementsNameMap["RobotsDiagram"]["TrikSetPainterWidth"] = tr("Painter Width");
	mElementsNameMap["RobotsDiagram"]["TrikSmile"] = tr("Smile");
	mElementsNameMap["RobotsDiagram"]["TrikSystem"] = tr("System Call");
	mElementsNameMap["RobotsDiagram"]["TrikV62AngularServo"] = tr("Angular Servo");
	mElementsNameMap["RobotsDiagram"]["TrikV62ClearEncoder"] = tr("Clear Encoder");
	mElementsNameMap["RobotsDiagram"]["TrikV62WaitForEncoder"] = tr("Wait for Encoder");
	mElementsNameMap["RobotsDiagram"]["TrikV6ClearEncoder"] = tr("Clear Encoder");
	mElementsNameMap["RobotsDiagram"]["TrikV6EnginesBackward"] = tr("Motors Backward");
	mElementsNameMap["RobotsDiagram"]["TrikV6EnginesForward"] = tr("Motors Forward");
	mElementsNameMap["RobotsDiagram"]["TrikV6EnginesStop"] = tr("Stop Motors");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForAccelerometer"] = tr("Wait for Accelerometer");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForButton"] = tr("Wait for Button");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForEncoder"] = tr("Wait for Encoder");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForGyroscope"] = tr("Wait for Gyroscope");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForIRDistance"] = tr("Wait for Infrared Distance");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForLight"] = tr("Wait for Light");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForMessage"] = tr("Wait for Message");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForMotion"] = tr("Wait for Motion");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForSonarDistance"] = tr("Wait for Ultrasonic Distance");
	mElementsNameMap["RobotsDiagram"]["TrikWaitForTouchSensor"] = tr("Wait for Touch Sensor");
	mElementsNameMap["RobotsDiagram"]["TrikWaitGamepadButton"] = tr("Wait Gamepad Button");
	mElementsNameMap["RobotsDiagram"]["TrikWaitGamepadConnect"] = tr("Wait for Gamepad Connect");
	mElementsNameMap["RobotsDiagram"]["TrikWaitGamepadDisconnect"] = tr("Wait for Gamepad Disconnect");
	mElementsNameMap["RobotsDiagram"]["TrikWaitGamepadWheel"] = tr("Wait for Gamepad Wheel");
	mElementsNameMap["RobotsDiagram"]["TrikWaitPadPress"] = tr("Wait Pad Press");
	mElementsNameMap["RobotsDiagram"]["TrikWriteToFile"] = tr("Write To File");
	mElementsNameMap["RobotsDiagram"]["VariableInit"] = tr("Variable Initialization");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["ClearScreen"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["CommentBlock"]["Comment"] = tr("Comment");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["ControlFlow"]["Guard"] = tr("Guard");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3Beep"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3Beep"]["WaitForCompletion"] = tr("Wait for Completion");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3ClearEncoder"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawCircle"]["CircleRadius"] = tr("Radius");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawCircle"]["Filled"] = tr("Filled");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawCircle"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawCircle"]["XCoordinateCircle"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawCircle"]["YCoordinateCircle"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawLine"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawLine"]["X1CoordinateLine"] = tr("X1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawLine"]["X2CoordinateLine"] = tr("X2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawLine"]["Y1CoordinateLine"] = tr("Y1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawLine"]["Y2CoordinateLine"] = tr("Y2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawPixel"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawPixel"]["XCoordinatePix"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawPixel"]["YCoordinatePix"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["Filled"] = tr("Filled");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["HeightRect"] = tr("Height");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["WidthRect"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["XCoordinateRect"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3DrawRect"]["YCoordinateRect"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesBackward"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesBackward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesBackward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesForward"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesForward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesForward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3EnginesStop"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3Led"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3PlayTone"]["Duration"] = tr("Duration");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3PlayTone"]["Frequency"] = tr("Frequency");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3PlayTone"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3PlayTone"]["WaitForCompletion"] = tr("Wait for Completion");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForButton"]["Button"] = tr("Button");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForColor"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForColor"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForColorIntensity"]["Intensity"] = tr("Intensity");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForColorIntensity"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForColorIntensity"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForEncoder"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForEncoder"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForEncoder"]["TachoLimit"] = tr("Tacho Limit");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForLight"]["Percents"] = tr("Percents");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForLight"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForLight"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSonarDistance"]["Distance"] = tr("Distance");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSonarDistance"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSonarDistance"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSound"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSound"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForSound"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Ev3WaitForTouchSensor"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Function"]["Body"] = tr("Body");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["GetButtonCode"]["Variable"] = tr("Variable");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["GetButtonCode"]["Wait"] = tr("Wait");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["IfBlock"]["Condition"] = tr("Condition");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["KillThread"]["Thread"] = tr("Thread");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Loop"]["Iterations"] = tr("Iterations");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["MarkerDown"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtBeep"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtBeep"]["WaitForCompletion"] = tr("Wait for Completion");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtClearEncoder"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawCircle"]["CircleRadius"] = tr("Radius");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawCircle"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawCircle"]["XCoordinateCircle"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawCircle"]["YCoordinateCircle"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawLine"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawLine"]["X1CoordinateLine"] = tr("X1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawLine"]["X2CoordinateLine"] = tr("X2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawLine"]["Y1CoordinateLine"] = tr("Y1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawLine"]["Y2CoordinateLine"] = tr("Y2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawPixel"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawPixel"]["XCoordinatePix"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawPixel"]["YCoordinatePix"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawRect"]["HeightRect"] = tr("Height");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawRect"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawRect"]["WidthRect"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawRect"]["XCoordinateRect"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtDrawRect"]["YCoordinateRect"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesBackward"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesBackward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesBackward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesForward"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesForward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesForward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtEnginesStop"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtPlayTone"]["Duration"] = tr("Duration");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtPlayTone"]["Frequency"] = tr("Frequency");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtPlayTone"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtPlayTone"]["WaitForCompletion"] = tr("Wait for Completion");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForButton"]["Button"] = tr("Button");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForColor"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForColor"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForColorIntensity"]["Intensity"] = tr("Intensity");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForColorIntensity"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForColorIntensity"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForEncoder"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForEncoder"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForEncoder"]["TachoLimit"] = tr("Tacho Limit");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForLight"]["Percents"] = tr("Percents");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForLight"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForLight"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSonarDistance"]["Distance"] = tr("Distance");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSonarDistance"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSonarDistance"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSound"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSound"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForSound"]["Volume"] = tr("Volume");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["NxtWaitForTouchSensor"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["PrintText"]["Evaluate"] = tr("Evaluate");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["PrintText"]["PrintText"] = tr("Text");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["PrintText"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["PrintText"]["XCoordinateText"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["PrintText"]["YCoordinateText"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Randomizer"]["LowerBound"] = tr("From");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Randomizer"]["UpperBound"] = tr("To");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Randomizer"]["Variable"] = tr("Variable");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["ReceiveMessageThreads"]["Synchronized"] = tr("Synchronized");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["ReceiveMessageThreads"]["Variable"] = tr("Variable");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["RobotsDiagramNode"]["devicesConfiguration"] = tr("Devices configuration");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["SendMessageThreads"]["Message"] = tr("Message");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["SendMessageThreads"]["Thread"] = tr("Thread");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["SwitchBlock"]["Expression"] = tr("Expression");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["Timer"]["Delay"] = tr("Delay (ms)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikAngularServo"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikAngularServo"]["Power"] = tr("Angle (degrees)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDetect"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDetectorToVariable"]["Variable"] = tr("Variable");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["HeightArc"] = tr("Height");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["SpanAngle"] = tr("SpanAngle");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["StartAngle"] = tr("StartAngle");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["WidthArc"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["XCoordinateArc"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawArc"]["YCoordinateArc"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawEllipse"]["HeightEllipse"] = tr("Height");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawEllipse"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawEllipse"]["WidthEllipse"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawEllipse"]["XCoordinateEllipse"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawEllipse"]["YCoordinateEllipse"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawLine"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawLine"]["X1CoordinateLine"] = tr("X1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawLine"]["X2CoordinateLine"] = tr("X2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawLine"]["Y1CoordinateLine"] = tr("Y1");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawLine"]["Y2CoordinateLine"] = tr("Y2");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawPixel"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawPixel"]["XCoordinatePix"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawPixel"]["YCoordinatePix"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawRect"]["HeightRect"] = tr("Height");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawRect"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawRect"]["WidthRect"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawRect"]["XCoordinateRect"] = tr("X");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikDrawRect"]["YCoordinateRect"] = tr("Y");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikInitCamera"]["Mode"] = tr("Mode");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikLed"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikPlayTone"]["FileName"] = tr("File Name");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikRemoveFile"]["File"] = tr("File");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSay"]["Evaluate"] = tr("Evaluate");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSay"]["Text"] = tr("Text");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSendMessage"]["HullNumber"] = tr("Hull Number");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSendMessage"]["Message"] = tr("Message");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSetBackground"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSetBackground"]["Redraw"] = tr("Redraw");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSetPainterColor"]["Color"] = tr("Color");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSetPainterWidth"]["Width"] = tr("Width");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSystem"]["Code"] = tr("Code");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSystem"]["Command"] = tr("Command");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikSystem"]["Evaluate"] = tr("Evaluate");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62AngularServo"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62AngularServo"]["Power"] = tr("Angle (degrees)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62ClearEncoder"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62WaitForEncoder"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62WaitForEncoder"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV62WaitForEncoder"]["TachoLimit"] = tr("Tacho Limit");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6ClearEncoder"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6EnginesBackward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6EnginesBackward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6EnginesForward"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6EnginesForward"]["Power"] = tr("Power (%)");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikV6EnginesStop"]["Ports"] = tr("Ports");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForAccelerometer"]["Acceleration"] = tr("Acceleration");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForAccelerometer"]["AccelerationAxis"] = tr("Acceleration Axis");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForAccelerometer"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForButton"]["Button"] = tr("Button");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForEncoder"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForEncoder"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForEncoder"]["TachoLimit"] = tr("Tacho Limit");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForGyroscope"]["Degrees"] = tr("Degrees/sec");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForGyroscope"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForIRDistance"]["Distance"] = tr("Distance");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForIRDistance"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForIRDistance"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForLight"]["Percents"] = tr("Percents");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForLight"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForLight"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForMessage"]["Synchronized"] = tr("Synchronized");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForMessage"]["Variable"] = tr("Variable");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForSonarDistance"]["Distance"] = tr("Distance");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForSonarDistance"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForSonarDistance"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitForTouchSensor"]["Port"] = tr("Port");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitGamepadButton"]["Button"] = tr("Button");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitGamepadWheel"]["Angle"] = tr("Angle");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitGamepadWheel"]["Sign"] = tr("Sign");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWaitPadPress"]["Pad"] = tr("Pad");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWriteToFile"]["File"] = tr("File");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["TrikWriteToFile"]["Text"] = tr("Text");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["VariableInit"]["value"] = tr("Value");
	mPropertiesDisplayedNamesMap["RobotsDiagram"]["VariableInit"]["variable"] = tr("Variable");
}

void RobotsMetamodelPlugin::initNodesAndEdgesSets()
{
	mNodes = QSet<QString>({
		"TrikSmile",
		"NxtDrawRect",
		"TrikRemoveFile",
		"TrikWaitForGyroscope",
		"Ev3WaitForLight",
		"TrikInitCamera",
		"TrikDetect",
		"TrikV62ClearEncoder",
		"Fork",
		"TrikLed",
		"Ev3WaitForSonarDistance",
		"NxtSensorBlock",
		"Ev3EngineMovementCommand",
		"NxtWaitForButton",
		"TrikWaitForTouchSensor",
		"TrikV6EnginesStop",
		"TrikWaitGamepadConnect",
		"Ev3SensorBlock",
		"Randomizer",
		"Ev3WaitForEncoder",
		"Ev3EnginesForward",
		"Timer",
		"TrikWaitGamepadButton",
		"FinalNode",
		"CommentBlock",
		"NxtEnginesForward",
		"TrikAnalogSensorBlock",
		"FiBlock",
		"NxtBeep",
		"TrikWaitForLight",
		"Ev3ClearEncoder",
		"MarkerDown",
		"NxtWaitForColor",
		"Ev3Led",
		"TrikWriteToFile",
		"EngineCommand",
		"Ev3WaitForColorIntensity",
		"TrikDetectorToVariable",
		"EngineMovementCommand",
		"TrikWaitGamepadWheel",
		"NxtWaitForEncoder",
		"NxtPlayTone",
		"TrikV62AngularServo",
		"Ev3WaitForButton",
		"NxtDrawLine",
		"Ev3DrawRect",
		"TrikSetBackground",
		"TrikDrawPixel",
		"ReceiveMessageThreads",
		"NxtWaitForSound",
		"TrikSendMessage",
		"TrikWaitPadPress",
		"Ev3DrawPixel",
		"TrikWaitForSonarDistance",
		"NxtDrawCircle",
		"NxtWaitForSonarDistance",
		"TrikSetPainterWidth",
		"NxtWaitForLight",
		"TrikSensorBlock",
		"TrikWaitForEncoder",
		"TrikWaitForButton",
		"GetButtonCode",
		"TrikDrawRect",
		"NxtEngineMovementCommand",
		"TrikDrawEllipse",
		"TrikWaitGamepadDisconnect",
		"NxtEnginesBackward",
		"Ev3EnginesBackward",
		"TrikWaitForIRDistance",
		"Join",
		"TrikInitVideoStreaming",
		"IfBlock",
		"NxtEnginesStop",
		"SwitchBlock",
		"TrikSay",
		"SendMessageThreads",
		"TrikSadSmile",
		"KillThread",
		"Ev3EnginesStop",
		"TrikWaitForMotion",
		"AbstractNode",
		"NxtWaitForTouchSensor",
		"InitialNode",
		"NxtDrawPixel",
		"VariableInit",
		"Ev3PlayTone",
		"TrikDigitalSensorBlock",
		"ClearScreen",
		"Ev3DrawLine",
		"TrikSystem",
		"SubprogramDiagram",
		"PrintText",
		"NxtClearEncoder",
		"NxtWaitForColorIntensity",
		"MarkerUp",
		"Function",
		"Ev3WaitForColor",
		"TrikV6EngineMovementCommand",
		"TrikAngularServo",
		"TrikV6ClearEncoder",
		"TrikV62WaitForEncoder",
		"Loop",
		"Ev3WaitForTouchSensor",
		"TrikWaitForAccelerometer",
		"TrikPlayTone",
		"RobotsDiagramNode",
		"TrikV6EnginesForward",
		"TrikDrawArc",
		"TrikDrawLine",
		"TrikWaitForMessage",
		"Ev3Beep",
		"Ev3WaitForSound",
		"Subprogram",
		"Ev3DrawCircle",
		"TrikSetPainterColor",
		"TrikV6EnginesBackward",
	});
	mEdges = QSet<QString>({
		"ControlFlow",
	});
}

void RobotsMetamodelPlugin::initPaletteGroupsMap()
{
	{
		QStringList groupElements;
		groupElements << QString::fromUtf8("InitialNode");
		groupElements << QString::fromUtf8("FinalNode");
		groupElements << QString::fromUtf8("Function");
		groupElements << QString::fromUtf8("IfBlock");
		groupElements << QString::fromUtf8("FiBlock");
		groupElements << QString::fromUtf8("SwitchBlock");
		groupElements << QString::fromUtf8("Loop");
		groupElements << QString::fromUtf8("Subprogram");
		groupElements << QString::fromUtf8("Fork");
		groupElements << QString::fromUtf8("Join");
		groupElements << QString::fromUtf8("KillThread");
		groupElements << QString::fromUtf8("VariableInit");
		groupElements << QString::fromUtf8("Randomizer");
		groupElements << QString::fromUtf8("CommentBlock");
		mPaletteGroupsMap[QString::fromUtf8("RobotsDiagram")].append(qMakePair(tr("Algorithms"), groupElements));
	}
	{
		QStringList groupElements;
		groupElements << QString::fromUtf8("NxtBeep");
		groupElements << QString::fromUtf8("NxtPlayTone");
		groupElements << QString::fromUtf8("NxtEnginesForward");
		groupElements << QString::fromUtf8("NxtEnginesBackward");
		groupElements << QString::fromUtf8("NxtEnginesStop");
		groupElements << QString::fromUtf8("NxtClearEncoder");
		groupElements << QString::fromUtf8("Ev3Beep");
		groupElements << QString::fromUtf8("Ev3PlayTone");
		groupElements << QString::fromUtf8("Ev3EnginesForward");
		groupElements << QString::fromUtf8("Ev3EnginesBackward");
		groupElements << QString::fromUtf8("Ev3EnginesStop");
		groupElements << QString::fromUtf8("Ev3ClearEncoder");
		groupElements << QString::fromUtf8("Ev3Led");
		groupElements << QString::fromUtf8("TrikPlayTone");
		groupElements << QString::fromUtf8("TrikV6EnginesForward");
		groupElements << QString::fromUtf8("TrikV6EnginesBackward");
		groupElements << QString::fromUtf8("TrikV6EnginesStop");
		groupElements << QString::fromUtf8("TrikV6ClearEncoder");
		groupElements << QString::fromUtf8("TrikV62ClearEncoder");
		groupElements << QString::fromUtf8("TrikAngularServo");
		groupElements << QString::fromUtf8("TrikV62AngularServo");
		groupElements << QString::fromUtf8("TrikSay");
		groupElements << QString::fromUtf8("TrikLed");
		groupElements << QString::fromUtf8("TrikSystem");
		groupElements << QString::fromUtf8("TrikInitCamera");
		groupElements << QString::fromUtf8("TrikDetect");
		groupElements << QString::fromUtf8("TrikDetectorToVariable");
		groupElements << QString::fromUtf8("TrikInitVideoStreaming");
		groupElements << QString::fromUtf8("TrikSendMessage");
		groupElements << QString::fromUtf8("SendMessageThreads");
		groupElements << QString::fromUtf8("TrikWriteToFile");
		groupElements << QString::fromUtf8("TrikRemoveFile");
		groupElements << QString::fromUtf8("GetButtonCode");
		mPaletteGroupsMap[QString::fromUtf8("RobotsDiagram")].append(qMakePair(tr("Actions"), groupElements));
	}
	{
		QStringList groupElements;
		groupElements << QString::fromUtf8("Timer");
		groupElements << QString::fromUtf8("Ev3WaitForTouchSensor");
		groupElements << QString::fromUtf8("Ev3WaitForEncoder");
		groupElements << QString::fromUtf8("Ev3WaitForColor");
		groupElements << QString::fromUtf8("Ev3WaitForColorIntensity");
		groupElements << QString::fromUtf8("Ev3WaitForLight");
		groupElements << QString::fromUtf8("Ev3WaitForSonarDistance");
		groupElements << QString::fromUtf8("Ev3WaitForButton");
		groupElements << QString::fromUtf8("Ev3WaitForSound");
		groupElements << QString::fromUtf8("NxtWaitForTouchSensor");
		groupElements << QString::fromUtf8("NxtWaitForSonarDistance");
		groupElements << QString::fromUtf8("NxtWaitForColor");
		groupElements << QString::fromUtf8("NxtWaitForEncoder");
		groupElements << QString::fromUtf8("NxtWaitForColorIntensity");
		groupElements << QString::fromUtf8("NxtWaitForLight");
		groupElements << QString::fromUtf8("NxtWaitForSound");
		groupElements << QString::fromUtf8("NxtWaitForButton");
		groupElements << QString::fromUtf8("TrikWaitForTouchSensor");
		groupElements << QString::fromUtf8("TrikWaitForLight");
		groupElements << QString::fromUtf8("TrikWaitForSonarDistance");
		groupElements << QString::fromUtf8("TrikWaitForIRDistance");
		groupElements << QString::fromUtf8("TrikWaitForEncoder");
		groupElements << QString::fromUtf8("TrikV62WaitForEncoder");
		groupElements << QString::fromUtf8("TrikWaitForButton");
		groupElements << QString::fromUtf8("TrikWaitForMessage");
		groupElements << QString::fromUtf8("TrikWaitGamepadButton");
		groupElements << QString::fromUtf8("TrikWaitPadPress");
		groupElements << QString::fromUtf8("TrikWaitGamepadWheel");
		groupElements << QString::fromUtf8("TrikWaitGamepadDisconnect");
		groupElements << QString::fromUtf8("TrikWaitGamepadConnect");
		groupElements << QString::fromUtf8("ReceiveMessageThreads");
		mPaletteGroupsMap[QString::fromUtf8("RobotsDiagram")].append(qMakePair(tr("Wait"), groupElements));
	}
	{
		QStringList groupElements;
		groupElements << QString::fromUtf8("NxtDrawRect");
		groupElements << QString::fromUtf8("NxtDrawPixel");
		groupElements << QString::fromUtf8("NxtDrawLine");
		groupElements << QString::fromUtf8("NxtDrawCircle");
		groupElements << QString::fromUtf8("Ev3DrawRect");
		groupElements << QString::fromUtf8("Ev3DrawPixel");
		groupElements << QString::fromUtf8("Ev3DrawLine");
		groupElements << QString::fromUtf8("Ev3DrawCircle");
		groupElements << QString::fromUtf8("TrikSetPainterColor");
		groupElements << QString::fromUtf8("TrikSetPainterWidth");
		groupElements << QString::fromUtf8("TrikDrawPixel");
		groupElements << QString::fromUtf8("TrikDrawLine");
		groupElements << QString::fromUtf8("TrikDrawRect");
		groupElements << QString::fromUtf8("TrikDrawEllipse");
		groupElements << QString::fromUtf8("TrikDrawArc");
		groupElements << QString::fromUtf8("TrikSmile");
		groupElements << QString::fromUtf8("TrikSadSmile");
		groupElements << QString::fromUtf8("TrikSetBackground");
		groupElements << QString::fromUtf8("PrintText");
		groupElements << QString::fromUtf8("ClearScreen");
		groupElements << QString::fromUtf8("MarkerDown");
		groupElements << QString::fromUtf8("MarkerUp");
		mPaletteGroupsMap[QString::fromUtf8("RobotsDiagram")].append(qMakePair(tr("Drawing"), groupElements));
	}
}

void RobotsMetamodelPlugin::initPaletteGroupsDescriptionMap()
{
}

void RobotsMetamodelPlugin::initMouseGestureMap()
{
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3Beep"] = "152, 12 : 281, 12 :  | 1"
			"52, 183 : 152, 12 :  | 155, 170 : 154, 176 : 153, 183 : 151, 190 : 149, 196 : 14"
			"6, 202 : 143, 208 : 139, 214 : 134, 219 : 129, 224 : 123, 228 : 117, 231 : 111, "
			"234 : 105, 236 : 98, 238 : 91, 239 : 85, 239 : 79, 239 : 72, 238 : 65, 236 : 59,"
			" 234 : 53, 231 : 47, 228 : 41, 224 : 36, 219 : 31, 214 : 27, 208 : 24, 202 : 21,"
			" 196 : 19, 190 : 17, 183 : 16, 176 : 16, 170 : 16, 164 : 17, 157 : 19, 150 : 21,"
			" 144 : 24, 138 : 27, 132 : 31, 126 : 36, 121 : 41, 116 : 47, 112 : 53, 109 : 59,"
			" 106 : 65, 104 : 72, 102 : 79, 101 : 85, 101 : 91, 101 : 98, 102 : 105, 104 : 11"
			"1, 106 : 117, 109 : 123, 112 : 129, 116 : 134, 121 : 139, 126 : 143, 132 : 146, "
			"138 : 149, 144 : 151, 150 : 153, 157 : 154, 164 : 155, 170 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3ClearEncoder"] = "112, 176 : 78, 1"
			"42 :  | 112, 176 : 148, 140 :  | 112, 33 : 112, 176 :  | 12, 12 : 210, 12 : 210,"
			" 210 : 12, 210 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3EnginesBackward"] = "111, 224 : 0,"
			" 113 :  | 113, 0 : 0, 113 :  | 0, 113 : 287, 113 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3EnginesForward"] = "185, 207 : 287"
			", 105 :  | 287, 105 : 182, 0 :  | 0, 105 : 287, 105 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3EnginesStop"] = "49, 117 : 185, 11"
			"7 :  | 223, 118 : 222, 128 : 221, 138 : 218, 148 : 215, 158 : 211, 167 : 205, 17"
			"6 : 199, 184 : 192, 192 : 184, 199 : 176, 205 : 167, 211 : 158, 215 : 148, 218 :"
			" 138, 221 : 128, 222 : 118, 223 : 108, 222 : 98, 221 : 88, 218 : 78, 215 : 69, 2"
			"11 : 60, 205 : 52, 199 : 44, 192 : 37, 184 : 31, 176 : 25, 167 : 21, 158 : 18, 1"
			"48 : 15, 138 : 14, 128 : 13, 118 : 14, 108 : 15, 98 : 18, 88 : 21, 78 : 25, 69 :"
			" 31, 60 : 37, 52 : 44, 44 : 52, 37 : 60, 31 : 69, 25 : 78, 21 : 88, 18 : 98, 15 "
			": 108, 14 : 118, 13 : 128, 14 : 138, 15 : 148, 18 : 158, 21 : 167, 25 : 176, 31 "
			": 184, 37 : 192, 44 : 199, 52 : 205, 60 : 211, 69 : 215, 78 : 218, 88 : 221, 98 "
			": 222, 108 : 223, 118 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3Led"] = "70, 79 : 70, 121 :  | 25,"
			" 78 : 25, 122 :  | 12, 12 : 86, 12 : 86, 78 : 12, 78 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3PlayTone"] = "130, 62 : 130, 146 :"
			"  | 106, 0 : 106, 212 :  | 0, 106 : 106, 212 :  | 0, 106 : 106, 0 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForColor"] = "196, 12 : 196, 1"
			"54 :  | 12, 12 : 380, 12 : 380, 154 : 12, 154 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForColorIntensity"] = "201, 13"
			"7 : 200, 143 : 199, 149 : 198, 154 : 196, 160 : 193, 166 : 190, 171 : 186, 176 :"
			" 182, 180 : 178, 184 : 173, 188 : 168, 191 : 162, 194 : 156, 196 : 151, 197 : 14"
			"5, 198 : 139, 198 : 133, 198 : 127, 197 : 122, 196 : 116, 194 : 110, 191 : 105, "
			"188 : 100, 184 : 96, 180 : 92, 176 : 88, 171 : 85, 166 : 82, 160 : 80, 154 : 79,"
			" 149 : 78, 143 : 78, 137 : 78, 131 : 79, 125 : 80, 120 : 82, 114 : 85, 108 : 88,"
			" 103 : 92, 98 : 96, 94 : 100, 90 : 105, 86 : 110, 83 : 116, 80 : 122, 78 : 127, "
			"77 : 133, 76 : 139, 76 : 145, 76 : 151, 77 : 156, 78 : 162, 80 : 168, 83 : 173, "
			"86 : 178, 90 : 182, 94 : 186, 98 : 190, 103 : 193, 108 : 196, 114 : 198, 120 : 1"
			"99, 125 : 200, 131 : 201, 137 :  | 260, 136 : 259, 148 : 257, 160 : 254, 171 : 2"
			"50, 183 : 245, 194 : 239, 204 : 231, 214 : 223, 223 : 214, 231 : 204, 239 : 194,"
			" 245 : 183, 250 : 171, 254 : 160, 257 : 148, 259 : 136, 259 : 124, 259 : 112, 25"
			"7 : 101, 254 : 89, 250 : 78, 245 : 68, 239 : 58, 231 : 49, 223 : 41, 214 : 33, 2"
			"04 : 27, 194 : 22, 183 : 18, 171 : 15, 160 : 13, 148 : 13, 136 : 13, 124 : 15, 1"
			"12 : 18, 101 : 22, 89 : 27, 78 : 33, 68 : 41, 58 : 49, 49 : 58, 41 : 68, 33 : 78"
			", 27 : 89, 22 : 101, 18 : 112, 15 : 124, 13 : 136, 13 : 148, 13 : 160, 15 : 171,"
			" 18 : 183, 22 : 194, 27 : 204, 33 : 214, 41 : 223, 49 : 231, 58 : 239, 68 : 245,"
			" 78 : 250, 89 : 254, 101 : 257, 112 : 259, 124 : 260, 136 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForEncoder"] = "203, 211 : 168"
			", 176 :  | 203, 211 : 240, 174 :  | 12, 48 : 50, 86 :  | 12, 48 : 48, 12 :  | 20"
			"3, 48 : 203, 211 :  | 12, 48 : 203, 48 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForLight"] = "249, 220 : 349, "
			"220 :  | 130, 12 : 130, 102 :  | 249, 221 : 248, 232 : 247, 244 : 244, 255 : 240"
			", 266 : 235, 276 : 229, 286 : 222, 296 : 214, 304 : 206, 312 : 196, 319 : 186, 3"
			"25 : 176, 330 : 165, 334 : 154, 337 : 142, 338 : 131, 339 : 120, 338 : 108, 337 "
			": 97, 334 : 86, 330 : 76, 325 : 66, 319 : 56, 312 : 48, 304 : 40, 296 : 33, 286 "
			": 27, 276 : 22, 266 : 18, 255 : 15, 244 : 14, 232 : 13, 221 : 14, 210 : 15, 198 "
			": 18, 187 : 22, 176 : 27, 166 : 33, 156 : 40, 146 : 48, 138 : 56, 130 : 66, 123 "
			": 76, 117 : 86, 112 : 97, 108 : 108, 105 : 120, 104 : 131, 103 : 142, 104 : 154,"
			" 105 : 165, 108 : 176, 112 : 186, 117 : 196, 123 : 206, 130 : 214, 138 : 222, 14"
			"6 : 229, 156 : 235, 166 : 240, 176 : 244, 187 : 247, 198 : 248, 210 : 249, 221 :"
			" ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForSonarDistance"] = "190, 162"
			" : 219, 122 :  | 219, 122 : 190, 86 :  | 120, 122 : 219, 122 :  | 12, 118 : 118,"
			" 12 :  | 12, 118 : 118, 224 :  | 118, 12 : 118, 224 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForSound"] = "12, 178 : 117, 1"
			"78 :  | 117, 96 : 117, 178 :  | 13, 96 : 117, 96 :  | 13, 12 : 13, 96 :  | 13, 1"
			"2 : 118, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Ev3WaitForTouchSensor"] = "12, 123 : "
			"174, 123 :  | 174, 12 : 358, 12 : 358, 234 : 174, 234 : 174, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["FinalNode"] = "13, 200 : 198, 15 :  |"
			" 12, 12 : 198, 198 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Fork"] = "13, 86 : 231, 86 :  | 12, 1"
			"2 : 231, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Function"] = "58, 12 : 58, 172 :  | 5"
			"8, 12 : 148, 12 :  | 12, 96 : 108, 96 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["IfBlock"] = "96, 182 : 180, 94 :  | 9"
			"8, 12 : 180, 94 :  | 12, 98 : 96, 182 :  | 98, 12 : 12, 98 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["InitialNode"] = "210, 111 : 209, 120 "
			": 208, 130 : 205, 139 : 202, 148 : 198, 157 : 193, 166 : 187, 173 : 181, 181 : 1"
			"73, 187 : 166, 193 : 157, 198 : 148, 202 : 139, 205 : 130, 208 : 120, 209 : 111,"
			" 209 : 102, 209 : 92, 208 : 83, 205 : 74, 202 : 65, 198 : 56, 193 : 49, 187 : 41"
			", 181 : 35, 173 : 29, 166 : 24, 157 : 20, 148 : 17, 139 : 14, 130 : 13, 120 : 13"
			", 111 : 13, 102 : 14, 92 : 17, 83 : 20, 74 : 24, 65 : 29, 56 : 35, 49 : 41, 41 :"
			" 49, 35 : 56, 29 : 65, 24 : 74, 20 : 83, 17 : 92, 14 : 102, 13 : 111, 13 : 120, "
			"13 : 130, 14 : 139, 17 : 148, 20 : 157, 24 : 166, 29 : 173, 35 : 181, 41 : 187, "
			"49 : 193, 56 : 198, 65 : 202, 74 : 205, 83 : 208, 92 : 209, 102 : 210, 111 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Loop"] = "154, 251 : 205, 302 :  | 15"
			"4, 251 : 209, 196 :  | 100, 12 : 154, 66 :  | 154, 66 : 103, 117 :  | 12, 66 : 2"
			"97, 66 : 297, 251 : 12, 251 : 12, 66 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtBeep"] = "152, 12 : 281, 12 :  | 1"
			"52, 183 : 152, 12 :  | 155, 170 : 154, 176 : 153, 183 : 151, 190 : 149, 196 : 14"
			"6, 202 : 143, 208 : 139, 214 : 134, 219 : 129, 224 : 123, 228 : 117, 231 : 111, "
			"234 : 105, 236 : 98, 238 : 91, 239 : 85, 239 : 79, 239 : 72, 238 : 65, 236 : 59,"
			" 234 : 53, 231 : 47, 228 : 41, 224 : 36, 219 : 31, 214 : 27, 208 : 24, 202 : 21,"
			" 196 : 19, 190 : 17, 183 : 16, 176 : 16, 170 : 16, 164 : 17, 157 : 19, 150 : 21,"
			" 144 : 24, 138 : 27, 132 : 31, 126 : 36, 121 : 41, 116 : 47, 112 : 53, 109 : 59,"
			" 106 : 65, 104 : 72, 102 : 79, 101 : 85, 101 : 91, 101 : 98, 102 : 105, 104 : 11"
			"1, 106 : 117, 109 : 123, 112 : 129, 116 : 134, 121 : 139, 126 : 143, 132 : 146, "
			"138 : 149, 144 : 151, 150 : 153, 157 : 154, 164 : 155, 170 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtClearEncoder"] = "112, 176 : 78, 1"
			"42 :  | 112, 176 : 148, 140 :  | 112, 33 : 112, 176 :  | 12, 12 : 210, 12 : 210,"
			" 210 : 12, 210 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtEnginesBackward"] = "111, 224 : 0,"
			" 113 :  | 113, 0 : 0, 113 :  | 0, 113 : 287, 113 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtEnginesForward"] = "185, 207 : 287"
			", 105 :  | 287, 105 : 182, 0 :  | 0, 105 : 287, 105 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtEnginesStop"] = "49, 117 : 185, 11"
			"7 :  | 223, 118 : 222, 128 : 221, 138 : 218, 148 : 215, 158 : 211, 167 : 205, 17"
			"6 : 199, 184 : 192, 192 : 184, 199 : 176, 205 : 167, 211 : 158, 215 : 148, 218 :"
			" 138, 221 : 128, 222 : 118, 223 : 108, 222 : 98, 221 : 88, 218 : 78, 215 : 69, 2"
			"11 : 60, 205 : 52, 199 : 44, 192 : 37, 184 : 31, 176 : 25, 167 : 21, 158 : 18, 1"
			"48 : 15, 138 : 14, 128 : 13, 118 : 14, 108 : 15, 98 : 18, 88 : 21, 78 : 25, 69 :"
			" 31, 60 : 37, 52 : 44, 44 : 52, 37 : 60, 31 : 69, 25 : 78, 21 : 88, 18 : 98, 15 "
			": 108, 14 : 118, 13 : 128, 14 : 138, 15 : 148, 18 : 158, 21 : 167, 25 : 176, 31 "
			": 184, 37 : 192, 44 : 199, 52 : 205, 60 : 211, 69 : 215, 78 : 218, 88 : 221, 98 "
			": 222, 108 : 223, 118 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtPlayTone"] = "130, 62 : 130, 146 :"
			"  | 106, 0 : 106, 212 :  | 0, 106 : 106, 212 :  | 0, 106 : 106, 0 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForColor"] = "196, 12 : 196, 1"
			"54 :  | 12, 12 : 380, 12 : 380, 154 : 12, 154 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForColorIntensity"] = "201, 13"
			"7 : 200, 143 : 199, 149 : 198, 154 : 196, 160 : 193, 166 : 190, 171 : 186, 176 :"
			" 182, 180 : 178, 184 : 173, 188 : 168, 191 : 162, 194 : 156, 196 : 151, 197 : 14"
			"5, 198 : 139, 198 : 133, 198 : 127, 197 : 122, 196 : 116, 194 : 110, 191 : 105, "
			"188 : 100, 184 : 96, 180 : 92, 176 : 88, 171 : 85, 166 : 82, 160 : 80, 154 : 79,"
			" 149 : 78, 143 : 78, 137 : 78, 131 : 79, 125 : 80, 120 : 82, 114 : 85, 108 : 88,"
			" 103 : 92, 98 : 96, 94 : 100, 90 : 105, 86 : 110, 83 : 116, 80 : 122, 78 : 127, "
			"77 : 133, 76 : 139, 76 : 145, 76 : 151, 77 : 156, 78 : 162, 80 : 168, 83 : 173, "
			"86 : 178, 90 : 182, 94 : 186, 98 : 190, 103 : 193, 108 : 196, 114 : 198, 120 : 1"
			"99, 125 : 200, 131 : 201, 137 :  | 260, 136 : 259, 148 : 257, 160 : 254, 171 : 2"
			"50, 183 : 245, 194 : 239, 204 : 231, 214 : 223, 223 : 214, 231 : 204, 239 : 194,"
			" 245 : 183, 250 : 171, 254 : 160, 257 : 148, 259 : 136, 259 : 124, 259 : 112, 25"
			"7 : 101, 254 : 89, 250 : 78, 245 : 68, 239 : 58, 231 : 49, 223 : 41, 214 : 33, 2"
			"04 : 27, 194 : 22, 183 : 18, 171 : 15, 160 : 13, 148 : 13, 136 : 13, 124 : 15, 1"
			"12 : 18, 101 : 22, 89 : 27, 78 : 33, 68 : 41, 58 : 49, 49 : 58, 41 : 68, 33 : 78"
			", 27 : 89, 22 : 101, 18 : 112, 15 : 124, 13 : 136, 13 : 148, 13 : 160, 15 : 171,"
			" 18 : 183, 22 : 194, 27 : 204, 33 : 214, 41 : 223, 49 : 231, 58 : 239, 68 : 245,"
			" 78 : 250, 89 : 254, 101 : 257, 112 : 259, 124 : 260, 136 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForEncoder"] = "203, 211 : 168"
			", 176 :  | 203, 211 : 240, 174 :  | 12, 48 : 50, 86 :  | 12, 48 : 48, 12 :  | 20"
			"3, 48 : 203, 211 :  | 12, 48 : 203, 48 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForLight"] = "249, 220 : 349, "
			"220 :  | 130, 12 : 130, 102 :  | 249, 221 : 248, 232 : 247, 244 : 244, 255 : 240"
			", 266 : 235, 276 : 229, 286 : 222, 296 : 214, 304 : 206, 312 : 196, 319 : 186, 3"
			"25 : 176, 330 : 165, 334 : 154, 337 : 142, 338 : 131, 339 : 120, 338 : 108, 337 "
			": 97, 334 : 86, 330 : 76, 325 : 66, 319 : 56, 312 : 48, 304 : 40, 296 : 33, 286 "
			": 27, 276 : 22, 266 : 18, 255 : 15, 244 : 14, 232 : 13, 221 : 14, 210 : 15, 198 "
			": 18, 187 : 22, 176 : 27, 166 : 33, 156 : 40, 146 : 48, 138 : 56, 130 : 66, 123 "
			": 76, 117 : 86, 112 : 97, 108 : 108, 105 : 120, 104 : 131, 103 : 142, 104 : 154,"
			" 105 : 165, 108 : 176, 112 : 186, 117 : 196, 123 : 206, 130 : 214, 138 : 222, 14"
			"6 : 229, 156 : 235, 166 : 240, 176 : 244, 187 : 247, 198 : 248, 210 : 249, 221 :"
			" ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForSonarDistance"] = "190, 162"
			" : 219, 122 :  | 219, 122 : 190, 86 :  | 120, 122 : 219, 122 :  | 12, 118 : 118,"
			" 12 :  | 12, 118 : 118, 224 :  | 118, 12 : 118, 224 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForSound"] = "12, 178 : 117, 1"
			"78 :  | 117, 96 : 117, 178 :  | 13, 96 : 117, 96 :  | 13, 12 : 13, 96 :  | 13, 1"
			"2 : 118, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["NxtWaitForTouchSensor"] = "12, 123 : "
			"174, 123 :  | 174, 12 : 358, 12 : 358, 234 : 174, 234 : 174, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["Timer"] = "12, 117 : 117, 118 :  | 11"
			"7, 12 : 117, 118 :  | 222, 117 : 221, 127 : 219, 137 : 217, 147 : 214, 157 : 209"
			", 166 : 204, 175 : 198, 183 : 191, 191 : 183, 198 : 175, 204 : 166, 209 : 157, 2"
			"14 : 147, 217 : 137, 219 : 127, 221 : 117, 221 : 107, 221 : 97, 219 : 87, 217 : "
			"77, 214 : 68, 209 : 59, 204 : 51, 198 : 43, 191 : 36, 183 : 30, 175 : 25, 166 : "
			"20, 157 : 17, 147 : 15, 137 : 13, 127 : 13, 117 : 13, 107 : 15, 97 : 17, 87 : 20"
			", 77 : 25, 68 : 30, 59 : 36, 51 : 43, 43 : 51, 36 : 59, 30 : 68, 25 : 77, 20 : 8"
			"7, 17 : 97, 15 : 107, 13 : 117, 13 : 127, 13 : 137, 15 : 147, 17 : 157, 20 : 166"
			", 25 : 175, 30 : 183, 36 : 191, 43 : 198, 51 : 204, 59 : 209, 68 : 214, 77 : 217"
			", 87 : 219, 97 : 221, 107 : 222, 117 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikLed"] = "70, 79 : 70, 121 :  | 25"
			", 78 : 25, 122 :  | 12, 12 : 86, 12 : 86, 78 : 12, 78 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikPlayTone"] = "130, 62 : 130, 146 "
			":  | 106, 0 : 106, 212 :  | 0, 106 : 106, 212 :  | 0, 106 : 106, 0 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikSadSmile"] = "156, 91 : 173, 110 "
			":  | 115, 73 : 156, 91 :  | 78, 73 : 114, 73 :  | 33, 85 : 78, 73 :  | 12, 106 :"
			" 33, 85 :  | 126, 12 : 126, 48 :  | 63, 12 : 63, 49 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikSmile"] = "161, 71 : 166, 57 :  |"
			" 149, 82 : 161, 70 :  | 113, 91 : 149, 83 :  | 67, 91 : 113, 91 :  | 29, 84 : 65"
			", 91 :  | 17, 74 : 28, 83 :  | 12, 59 : 17, 73 :  | 130, 13 : 130, 51 :  | 62, 1"
			"2 : 62, 51 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV62ClearEncoder"] = "112, 176 : 7"
			"8, 142 :  | 112, 176 : 148, 140 :  | 112, 33 : 112, 176 :  | 12, 12 : 210, 12 : "
			"210, 210 : 12, 210 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV62WaitForEncoder"] = "203, 211 :"
			" 168, 176 :  | 203, 211 : 240, 174 :  | 12, 48 : 50, 86 :  | 12, 48 : 48, 12 :  "
			"| 203, 48 : 203, 211 :  | 12, 48 : 203, 48 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV6ClearEncoder"] = "112, 176 : 78"
			", 142 :  | 112, 176 : 148, 140 :  | 112, 33 : 112, 176 :  | 12, 12 : 210, 12 : 2"
			"10, 210 : 12, 210 : 12, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV6EnginesBackward"] = "111, 224 :"
			" 0, 113 :  | 113, 0 : 0, 113 :  | 0, 113 : 287, 113 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV6EnginesForward"] = "185, 207 : "
			"287, 105 :  | 287, 105 : 182, 0 :  | 0, 105 : 287, 105 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikV6EnginesStop"] = "49, 117 : 185,"
			" 117 :  | 223, 118 : 222, 128 : 221, 138 : 218, 148 : 215, 158 : 211, 167 : 205,"
			" 176 : 199, 184 : 192, 192 : 184, 199 : 176, 205 : 167, 211 : 158, 215 : 148, 21"
			"8 : 138, 221 : 128, 222 : 118, 223 : 108, 222 : 98, 221 : 88, 218 : 78, 215 : 69"
			", 211 : 60, 205 : 52, 199 : 44, 192 : 37, 184 : 31, 176 : 25, 167 : 21, 158 : 18"
			", 148 : 15, 138 : 14, 128 : 13, 118 : 14, 108 : 15, 98 : 18, 88 : 21, 78 : 25, 6"
			"9 : 31, 60 : 37, 52 : 44, 44 : 52, 37 : 60, 31 : 69, 25 : 78, 21 : 88, 18 : 98, "
			"15 : 108, 14 : 118, 13 : 128, 14 : 138, 15 : 148, 18 : 158, 21 : 167, 25 : 176, "
			"31 : 184, 37 : 192, 44 : 199, 52 : 205, 60 : 211, 69 : 215, 78 : 218, 88 : 221, "
			"98 : 222, 108 : 223, 118 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikWaitForEncoder"] = "203, 211 : 16"
			"8, 176 :  | 203, 211 : 240, 174 :  | 12, 48 : 50, 86 :  | 12, 48 : 48, 12 :  | 2"
			"03, 48 : 203, 211 :  | 12, 48 : 203, 48 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikWaitForGyroscope"] = "101, 106 : "
			"212, 106 :  | 156, 183 : 156, 107 :  | 12, 183 : 156, 183 :  | 12, 12 : 12, 183 "
			":  | 12, 12 : 164, 12 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikWaitForIRDistance"] = "190, 162 :"
			" 219, 122 :  | 219, 122 : 190, 86 :  | 120, 122 : 219, 122 :  | 12, 118 : 118, 1"
			"2 :  | 12, 118 : 118, 224 :  | 118, 12 : 118, 224 : ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikWaitForLight"] = "249, 220 : 349,"
			" 220 :  | 130, 12 : 130, 102 :  | 249, 221 : 248, 232 : 247, 244 : 244, 255 : 24"
			"0, 266 : 235, 276 : 229, 286 : 222, 296 : 214, 304 : 206, 312 : 196, 319 : 186, "
			"325 : 176, 330 : 165, 334 : 154, 337 : 142, 338 : 131, 339 : 120, 338 : 108, 337"
			" : 97, 334 : 86, 330 : 76, 325 : 66, 319 : 56, 312 : 48, 304 : 40, 296 : 33, 286"
			" : 27, 276 : 22, 266 : 18, 255 : 15, 244 : 14, 232 : 13, 221 : 14, 210 : 15, 198"
			" : 18, 187 : 22, 176 : 27, 166 : 33, 156 : 40, 146 : 48, 138 : 56, 130 : 66, 123"
			" : 76, 117 : 86, 112 : 97, 108 : 108, 105 : 120, 104 : 131, 103 : 142, 104 : 154"
			", 105 : 165, 108 : 176, 112 : 186, 117 : 196, 123 : 206, 130 : 214, 138 : 222, 1"
			"46 : 229, 156 : 235, 166 : 240, 176 : 244, 187 : 247, 198 : 248, 210 : 249, 221 "
			": ";
	mElementMouseGesturesMap["RobotsDiagram"]["TrikWaitForTouchSensor"] = "12, 123 :"
			" 174, 123 :  | 174, 12 : 358, 12 : 358, 234 : 174, 234 : 174, 12 : ";
}

void RobotsMetamodelPlugin::initPropertyMap()
{
	mPropertyTypes["ClearScreen"]["Redraw"] = "Bool";
	mPropertyTypes["ControlFlow"]["Guard"] = "GuardType";
	mPropertyTypes["Ev3Beep"]["WaitForCompletion"] = "Bool";
	mPropertyTypes["Ev3DrawCircle"]["CircleRadius"] = "Int";
	mPropertyTypes["Ev3DrawCircle"]["Filled"] = "Bool";
	mPropertyTypes["Ev3DrawCircle"]["Redraw"] = "Bool";
	mPropertyTypes["Ev3DrawCircle"]["XCoordinateCircle"] = "Int";
	mPropertyTypes["Ev3DrawCircle"]["YCoordinateCircle"] = "Int";
	mPropertyTypes["Ev3DrawLine"]["Redraw"] = "Bool";
	mPropertyTypes["Ev3DrawLine"]["X1CoordinateLine"] = "Int";
	mPropertyTypes["Ev3DrawLine"]["X2CoordinateLine"] = "Int";
	mPropertyTypes["Ev3DrawLine"]["Y1CoordinateLine"] = "Int";
	mPropertyTypes["Ev3DrawLine"]["Y2CoordinateLine"] = "Int";
	mPropertyTypes["Ev3DrawPixel"]["Redraw"] = "Bool";
	mPropertyTypes["Ev3DrawPixel"]["XCoordinatePix"] = "Int";
	mPropertyTypes["Ev3DrawPixel"]["YCoordinatePix"] = "Int";
	mPropertyTypes["Ev3DrawRect"]["Filled"] = "Bool";
	mPropertyTypes["Ev3DrawRect"]["HeightRect"] = "Int";
	mPropertyTypes["Ev3DrawRect"]["Redraw"] = "Bool";
	mPropertyTypes["Ev3DrawRect"]["WidthRect"] = "Int";
	mPropertyTypes["Ev3DrawRect"]["XCoordinateRect"] = "Int";
	mPropertyTypes["Ev3DrawRect"]["YCoordinateRect"] = "Int";
	mPropertyTypes["Ev3EnginesBackward"]["Mode"] = "BreakEngineMode";
	mPropertyTypes["Ev3EnginesForward"]["Mode"] = "BreakEngineMode";
	mPropertyTypes["Ev3Led"]["Color"] = "Ev3LedColor";
	mPropertyTypes["Ev3PlayTone"]["WaitForCompletion"] = "Bool";
	mPropertyTypes["Ev3WaitForButton"]["Button"] = "Ev3Buttons";
	mPropertyTypes["Ev3WaitForColor"]["Color"] = "Color";
	mPropertyTypes["Ev3WaitForColor"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["Ev3WaitForColorIntensity"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["Ev3WaitForColorIntensity"]["Sign"] = "DistanceSign";
	mPropertyTypes["Ev3WaitForEncoder"]["Port"] = "Ev3MotorPort";
	mPropertyTypes["Ev3WaitForEncoder"]["Sign"] = "DistanceSign";
	mPropertyTypes["Ev3WaitForLight"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["Ev3WaitForLight"]["Sign"] = "DistanceSign";
	mPropertyTypes["Ev3WaitForSonarDistance"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["Ev3WaitForSonarDistance"]["Sign"] = "DistanceSign";
	mPropertyTypes["Ev3WaitForSound"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["Ev3WaitForSound"]["Sign"] = "DistanceSign";
	mPropertyTypes["Ev3WaitForTouchSensor"]["Port"] = "Ev3SensorPort";
	mPropertyTypes["GetButtonCode"]["Wait"] = "Bool";
	mPropertyTypes["MarkerDown"]["Color"] = "BackgroundColor";
	mPropertyTypes["NxtBeep"]["WaitForCompletion"] = "Bool";
	mPropertyTypes["NxtDrawCircle"]["CircleRadius"] = "Int";
	mPropertyTypes["NxtDrawCircle"]["Redraw"] = "Bool";
	mPropertyTypes["NxtDrawCircle"]["XCoordinateCircle"] = "Int";
	mPropertyTypes["NxtDrawCircle"]["YCoordinateCircle"] = "Int";
	mPropertyTypes["NxtDrawLine"]["Redraw"] = "Bool";
	mPropertyTypes["NxtDrawLine"]["X1CoordinateLine"] = "Int";
	mPropertyTypes["NxtDrawLine"]["X2CoordinateLine"] = "Int";
	mPropertyTypes["NxtDrawLine"]["Y1CoordinateLine"] = "Int";
	mPropertyTypes["NxtDrawLine"]["Y2CoordinateLine"] = "Int";
	mPropertyTypes["NxtDrawPixel"]["Redraw"] = "Bool";
	mPropertyTypes["NxtDrawPixel"]["XCoordinatePix"] = "Int";
	mPropertyTypes["NxtDrawPixel"]["YCoordinatePix"] = "Int";
	mPropertyTypes["NxtDrawRect"]["HeightRect"] = "Int";
	mPropertyTypes["NxtDrawRect"]["Redraw"] = "Bool";
	mPropertyTypes["NxtDrawRect"]["WidthRect"] = "Int";
	mPropertyTypes["NxtDrawRect"]["XCoordinateRect"] = "Int";
	mPropertyTypes["NxtDrawRect"]["YCoordinateRect"] = "Int";
	mPropertyTypes["NxtEnginesBackward"]["Mode"] = "BreakEngineMode";
	mPropertyTypes["NxtEnginesForward"]["Mode"] = "BreakEngineMode";
	mPropertyTypes["NxtPlayTone"]["WaitForCompletion"] = "Bool";
	mPropertyTypes["NxtWaitForButton"]["Button"] = "NxtButtons";
	mPropertyTypes["NxtWaitForColor"]["Color"] = "Color";
	mPropertyTypes["NxtWaitForColor"]["Port"] = "NxtSensorPort";
	mPropertyTypes["NxtWaitForColorIntensity"]["Port"] = "NxtSensorPort";
	mPropertyTypes["NxtWaitForColorIntensity"]["Sign"] = "DistanceSign";
	mPropertyTypes["NxtWaitForEncoder"]["Sign"] = "DistanceSign";
	mPropertyTypes["NxtWaitForLight"]["Port"] = "NxtSensorPort";
	mPropertyTypes["NxtWaitForLight"]["Sign"] = "DistanceSign";
	mPropertyTypes["NxtWaitForSonarDistance"]["Port"] = "NxtSensorPort";
	mPropertyTypes["NxtWaitForSonarDistance"]["Sign"] = "DistanceSign";
	mPropertyTypes["NxtWaitForSound"]["Port"] = "NxtSensorPort";
	mPropertyTypes["NxtWaitForSound"]["Sign"] = "DistanceSign";
	mPropertyTypes["NxtWaitForTouchSensor"]["Port"] = "NxtSensorPort";
	mPropertyTypes["PrintText"]["Evaluate"] = "Bool";
	mPropertyTypes["PrintText"]["Redraw"] = "Bool";
	mPropertyTypes["ReceiveMessageThreads"]["Synchronized"] = "Bool";
	mPropertyTypes["TrikDetect"]["Mode"] = "DetectableVideoSensor";
	mPropertyTypes["TrikDrawArc"]["Redraw"] = "Bool";
	mPropertyTypes["TrikDrawArc"]["SpanAngle"] = "Int";
	mPropertyTypes["TrikDrawArc"]["StartAngle"] = "Int";
	mPropertyTypes["TrikDrawEllipse"]["Redraw"] = "Bool";
	mPropertyTypes["TrikDrawLine"]["Redraw"] = "Bool";
	mPropertyTypes["TrikDrawPixel"]["Redraw"] = "Bool";
	mPropertyTypes["TrikDrawRect"]["Redraw"] = "Bool";
	mPropertyTypes["TrikInitCamera"]["Mode"] = "VideoSensor";
	mPropertyTypes["TrikLed"]["Color"] = "TrikLedColor";
	mPropertyTypes["TrikSay"]["Evaluate"] = "Bool";
	mPropertyTypes["TrikSetBackground"]["Color"] = "BackgroundColor";
	mPropertyTypes["TrikSetBackground"]["Redraw"] = "Bool";
	mPropertyTypes["TrikSetPainterColor"]["Color"] = "BackgroundColor";
	mPropertyTypes["TrikSystem"]["Code"] = "Bool";
	mPropertyTypes["TrikSystem"]["Evaluate"] = "Bool";
	mPropertyTypes["TrikV62WaitForEncoder"]["Port"] = "TrikV62EncoderPort";
	mPropertyTypes["TrikV62WaitForEncoder"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForAccelerometer"]["AccelerationAxis"] = "Accelerations";
	mPropertyTypes["TrikWaitForAccelerometer"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForButton"]["Button"] = "TrikButtons";
	mPropertyTypes["TrikWaitForEncoder"]["Port"] = "TrikV6EncoderPort";
	mPropertyTypes["TrikWaitForEncoder"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForGyroscope"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForIRDistance"]["Port"] = "TrikV6AnalogSensorPort";
	mPropertyTypes["TrikWaitForIRDistance"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForLight"]["Port"] = "TrikV6AnalogSensorPort";
	mPropertyTypes["TrikWaitForLight"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForMessage"]["Synchronized"] = "Bool";
	mPropertyTypes["TrikWaitForSonarDistance"]["Port"] = "TrikV6DigitalSensorPort";
	mPropertyTypes["TrikWaitForSonarDistance"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitForTouchSensor"]["Port"] = "TrikV6AnalogSensorPort";
	mPropertyTypes["TrikWaitGamepadButton"]["Button"] = "Int";
	mPropertyTypes["TrikWaitGamepadWheel"]["Sign"] = "DistanceSign";
	mPropertyTypes["TrikWaitPadPress"]["Pad"] = "Int";
}

void RobotsMetamodelPlugin::initPropertyDefaultsMap()
{
	mPropertyDefault["ClearScreen"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["CommentBlock"]["Comment"] = tr("Enter some text here...");
	mPropertyDefault["Ev3Beep"]["Volume"] = tr("50");
	mPropertyDefault["Ev3Beep"]["WaitForCompletion"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3ClearEncoder"]["Ports"] = tr("A, B, C, D");
	mPropertyDefault["Ev3DrawCircle"]["CircleRadius"] = QString::fromUtf8("40");
	mPropertyDefault["Ev3DrawCircle"]["Filled"] = QString::fromUtf8("false");
	mPropertyDefault["Ev3DrawCircle"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3DrawCircle"]["XCoordinateCircle"] = QString::fromUtf8("85");
	mPropertyDefault["Ev3DrawCircle"]["YCoordinateCircle"] = QString::fromUtf8("70");
	mPropertyDefault["Ev3DrawLine"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3DrawLine"]["X1CoordinateLine"] = QString::fromUtf8("20");
	mPropertyDefault["Ev3DrawLine"]["X2CoordinateLine"] = QString::fromUtf8("150");
	mPropertyDefault["Ev3DrawLine"]["Y1CoordinateLine"] = QString::fromUtf8("90");
	mPropertyDefault["Ev3DrawLine"]["Y2CoordinateLine"] = QString::fromUtf8("20");
	mPropertyDefault["Ev3DrawPixel"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3DrawPixel"]["XCoordinatePix"] = QString::fromUtf8("90");
	mPropertyDefault["Ev3DrawPixel"]["YCoordinatePix"] = QString::fromUtf8("70");
	mPropertyDefault["Ev3DrawRect"]["Filled"] = QString::fromUtf8("false");
	mPropertyDefault["Ev3DrawRect"]["HeightRect"] = QString::fromUtf8("30");
	mPropertyDefault["Ev3DrawRect"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3DrawRect"]["WidthRect"] = QString::fromUtf8("40");
	mPropertyDefault["Ev3DrawRect"]["XCoordinateRect"] = QString::fromUtf8("75");
	mPropertyDefault["Ev3DrawRect"]["YCoordinateRect"] = QString::fromUtf8("50");
	mPropertyDefault["Ev3EnginesBackward"]["Mode"] = QString::fromUtf8("break");
	mPropertyDefault["Ev3EnginesBackward"]["Ports"] = tr("B, C");
	mPropertyDefault["Ev3EnginesBackward"]["Power"] = tr("100");
	mPropertyDefault["Ev3EnginesForward"]["Mode"] = QString::fromUtf8("break");
	mPropertyDefault["Ev3EnginesForward"]["Ports"] = tr("B, C");
	mPropertyDefault["Ev3EnginesForward"]["Power"] = tr("100");
	mPropertyDefault["Ev3EnginesStop"]["Ports"] = tr("A, B, C, D");
	mPropertyDefault["Ev3Led"]["Color"] = QString::fromUtf8("red");
	mPropertyDefault["Ev3PlayTone"]["Duration"] = tr("1000");
	mPropertyDefault["Ev3PlayTone"]["Frequency"] = tr("1000");
	mPropertyDefault["Ev3PlayTone"]["Volume"] = tr("50");
	mPropertyDefault["Ev3PlayTone"]["WaitForCompletion"] = QString::fromUtf8("true");
	mPropertyDefault["Ev3WaitForButton"]["Button"] = QString::fromUtf8("Up");
	mPropertyDefault["Ev3WaitForColor"]["Color"] = QString::fromUtf8("red");
	mPropertyDefault["Ev3WaitForColor"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["Ev3WaitForColorIntensity"]["Intensity"] = tr("0");
	mPropertyDefault["Ev3WaitForColorIntensity"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["Ev3WaitForColorIntensity"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["Ev3WaitForEncoder"]["Port"] = QString::fromUtf8("B");
	mPropertyDefault["Ev3WaitForEncoder"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["Ev3WaitForLight"]["Percents"] = tr("0");
	mPropertyDefault["Ev3WaitForLight"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["Ev3WaitForLight"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["Ev3WaitForSonarDistance"]["Distance"] = tr("0");
	mPropertyDefault["Ev3WaitForSonarDistance"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["Ev3WaitForSonarDistance"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["Ev3WaitForSound"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["Ev3WaitForSound"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["Ev3WaitForSound"]["Volume"] = tr("0");
	mPropertyDefault["Ev3WaitForTouchSensor"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["GetButtonCode"]["Variable"] = tr("x");
	mPropertyDefault["GetButtonCode"]["Wait"] = QString::fromUtf8("true");
	mPropertyDefault["IfBlock"]["Condition"] = tr("x > 0");
	mPropertyDefault["KillThread"]["Thread"] = tr("main");
	mPropertyDefault["Loop"]["Iterations"] = tr("10");
	mPropertyDefault["MarkerDown"]["Color"] = QString::fromUtf8("blue");
	mPropertyDefault["NxtBeep"]["Volume"] = tr("50");
	mPropertyDefault["NxtBeep"]["WaitForCompletion"] = QString::fromUtf8("true");
	mPropertyDefault["NxtClearEncoder"]["Ports"] = tr("A, B, C");
	mPropertyDefault["NxtDrawCircle"]["CircleRadius"] = QString::fromUtf8("5");
	mPropertyDefault["NxtDrawCircle"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["NxtDrawCircle"]["XCoordinateCircle"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawCircle"]["YCoordinateCircle"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawLine"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["NxtDrawLine"]["X1CoordinateLine"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawLine"]["X2CoordinateLine"] = QString::fromUtf8("10");
	mPropertyDefault["NxtDrawLine"]["Y1CoordinateLine"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawLine"]["Y2CoordinateLine"] = QString::fromUtf8("10");
	mPropertyDefault["NxtDrawPixel"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["NxtDrawPixel"]["XCoordinatePix"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawPixel"]["YCoordinatePix"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawRect"]["HeightRect"] = QString::fromUtf8("5");
	mPropertyDefault["NxtDrawRect"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["NxtDrawRect"]["WidthRect"] = QString::fromUtf8("5");
	mPropertyDefault["NxtDrawRect"]["XCoordinateRect"] = QString::fromUtf8("0");
	mPropertyDefault["NxtDrawRect"]["YCoordinateRect"] = QString::fromUtf8("0");
	mPropertyDefault["NxtEnginesBackward"]["Mode"] = QString::fromUtf8("break");
	mPropertyDefault["NxtEnginesBackward"]["Ports"] = tr("B, C");
	mPropertyDefault["NxtEnginesBackward"]["Power"] = tr("100");
	mPropertyDefault["NxtEnginesForward"]["Mode"] = QString::fromUtf8("break");
	mPropertyDefault["NxtEnginesForward"]["Ports"] = tr("B, C");
	mPropertyDefault["NxtEnginesForward"]["Power"] = tr("100");
	mPropertyDefault["NxtEnginesStop"]["Ports"] = tr("A, B, C");
	mPropertyDefault["NxtPlayTone"]["Duration"] = tr("1000");
	mPropertyDefault["NxtPlayTone"]["Frequency"] = tr("1000");
	mPropertyDefault["NxtPlayTone"]["Volume"] = tr("50");
	mPropertyDefault["NxtPlayTone"]["WaitForCompletion"] = QString::fromUtf8("true");
	mPropertyDefault["NxtWaitForButton"]["Button"] = QString::fromUtf8("Enter");
	mPropertyDefault["NxtWaitForColor"]["Color"] = QString::fromUtf8("red");
	mPropertyDefault["NxtWaitForColor"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["NxtWaitForColorIntensity"]["Intensity"] = tr("0");
	mPropertyDefault["NxtWaitForColorIntensity"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["NxtWaitForColorIntensity"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["NxtWaitForEncoder"]["Port"] = tr("B");
	mPropertyDefault["NxtWaitForEncoder"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["NxtWaitForLight"]["Percents"] = tr("0");
	mPropertyDefault["NxtWaitForLight"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["NxtWaitForLight"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["NxtWaitForSonarDistance"]["Distance"] = tr("0");
	mPropertyDefault["NxtWaitForSonarDistance"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["NxtWaitForSonarDistance"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["NxtWaitForSound"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["NxtWaitForSound"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["NxtWaitForSound"]["Volume"] = tr("0");
	mPropertyDefault["NxtWaitForTouchSensor"]["Port"] = QString::fromUtf8("1");
	mPropertyDefault["PrintText"]["Evaluate"] = QString::fromUtf8("false");
	mPropertyDefault["PrintText"]["PrintText"] = tr("Enter some text here");
	mPropertyDefault["PrintText"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["PrintText"]["XCoordinateText"] = tr("1");
	mPropertyDefault["PrintText"]["YCoordinateText"] = tr("1");
	mPropertyDefault["Randomizer"]["LowerBound"] = tr("0");
	mPropertyDefault["Randomizer"]["UpperBound"] = tr("10");
	mPropertyDefault["Randomizer"]["Variable"] = tr("x");
	mPropertyDefault["ReceiveMessageThreads"]["Synchronized"] = QString::fromUtf8("true");
	mPropertyDefault["SwitchBlock"]["Expression"] = tr("x");
	mPropertyDefault["Timer"]["Delay"] = tr("1000");
	mPropertyDefault["TrikAngularServo"]["Ports"] = tr("E1");
	mPropertyDefault["TrikAngularServo"]["Power"] = tr("0");
	mPropertyDefault["TrikDetect"]["Mode"] = QString::fromUtf8("line");
	mPropertyDefault["TrikDetectorToVariable"]["Variable"] = tr("err");
	mPropertyDefault["TrikDrawArc"]["HeightArc"] = tr("5");
	mPropertyDefault["TrikDrawArc"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikDrawArc"]["SpanAngle"] = QString::fromUtf8("20");
	mPropertyDefault["TrikDrawArc"]["StartAngle"] = QString::fromUtf8("20");
	mPropertyDefault["TrikDrawArc"]["WidthArc"] = tr("5");
	mPropertyDefault["TrikDrawArc"]["XCoordinateArc"] = tr("0");
	mPropertyDefault["TrikDrawArc"]["YCoordinateArc"] = tr("0");
	mPropertyDefault["TrikDrawEllipse"]["HeightEllipse"] = tr("5");
	mPropertyDefault["TrikDrawEllipse"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikDrawEllipse"]["WidthEllipse"] = tr("5");
	mPropertyDefault["TrikDrawEllipse"]["XCoordinateEllipse"] = tr("0");
	mPropertyDefault["TrikDrawEllipse"]["YCoordinateEllipse"] = tr("0");
	mPropertyDefault["TrikDrawLine"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikDrawLine"]["X1CoordinateLine"] = tr("0");
	mPropertyDefault["TrikDrawLine"]["X2CoordinateLine"] = tr("10");
	mPropertyDefault["TrikDrawLine"]["Y1CoordinateLine"] = tr("0");
	mPropertyDefault["TrikDrawLine"]["Y2CoordinateLine"] = tr("10");
	mPropertyDefault["TrikDrawPixel"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikDrawPixel"]["XCoordinatePix"] = tr("5");
	mPropertyDefault["TrikDrawPixel"]["YCoordinatePix"] = tr("5");
	mPropertyDefault["TrikDrawRect"]["HeightRect"] = tr("5");
	mPropertyDefault["TrikDrawRect"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikDrawRect"]["WidthRect"] = tr("5");
	mPropertyDefault["TrikDrawRect"]["XCoordinateRect"] = tr("0");
	mPropertyDefault["TrikDrawRect"]["YCoordinateRect"] = tr("0");
	mPropertyDefault["TrikInitCamera"]["Mode"] = QString::fromUtf8("line");
	mPropertyDefault["TrikLed"]["Color"] = QString::fromUtf8("red");
	mPropertyDefault["TrikPlayTone"]["FileName"] = tr("media/beep.wav");
	mPropertyDefault["TrikRemoveFile"]["File"] = tr("output.txt");
	mPropertyDefault["TrikSay"]["Evaluate"] = QString::fromUtf8("false");
	mPropertyDefault["TrikSay"]["Text"] = tr("Hi!");
	mPropertyDefault["TrikSetBackground"]["Color"] = QString::fromUtf8("white");
	mPropertyDefault["TrikSetBackground"]["Redraw"] = QString::fromUtf8("true");
	mPropertyDefault["TrikSetPainterColor"]["Color"] = QString::fromUtf8("black");
	mPropertyDefault["TrikSetPainterWidth"]["Width"] = tr("1");
	mPropertyDefault["TrikSystem"]["Code"] = QString::fromUtf8("false");
	mPropertyDefault["TrikSystem"]["Command"] = tr("echo 123");
	mPropertyDefault["TrikSystem"]["Evaluate"] = QString::fromUtf8("false");
	mPropertyDefault["TrikV62AngularServo"]["Ports"] = tr("S1");
	mPropertyDefault["TrikV62AngularServo"]["Power"] = tr("0");
	mPropertyDefault["TrikV62ClearEncoder"]["Ports"] = tr("E1, E2, E3, E4");
	mPropertyDefault["TrikV62WaitForEncoder"]["Port"] = QString::fromUtf8("E3");
	mPropertyDefault["TrikV62WaitForEncoder"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikV6ClearEncoder"]["Ports"] = tr("B1, B2, B3, B4");
	mPropertyDefault["TrikV6EnginesBackward"]["Ports"] = tr("M3, M4");
	mPropertyDefault["TrikV6EnginesBackward"]["Power"] = tr("100");
	mPropertyDefault["TrikV6EnginesForward"]["Ports"] = tr("M3, M4");
	mPropertyDefault["TrikV6EnginesForward"]["Power"] = tr("100");
	mPropertyDefault["TrikV6EnginesStop"]["Ports"] = tr("M1, M2, M3, M4");
	mPropertyDefault["TrikWaitForAccelerometer"]["Acceleration"] = tr("0");
	mPropertyDefault["TrikWaitForAccelerometer"]["AccelerationAxis"] = QString::fromUtf8("xAxis");
	mPropertyDefault["TrikWaitForAccelerometer"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForButton"]["Button"] = QString::fromUtf8("Up");
	mPropertyDefault["TrikWaitForEncoder"]["Port"] = QString::fromUtf8("B3");
	mPropertyDefault["TrikWaitForEncoder"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForGyroscope"]["Degrees"] = tr("0");
	mPropertyDefault["TrikWaitForGyroscope"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForIRDistance"]["Distance"] = tr("0");
	mPropertyDefault["TrikWaitForIRDistance"]["Port"] = QString::fromUtf8("A1");
	mPropertyDefault["TrikWaitForIRDistance"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForLight"]["Percents"] = tr("0");
	mPropertyDefault["TrikWaitForLight"]["Port"] = QString::fromUtf8("A1");
	mPropertyDefault["TrikWaitForLight"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForMessage"]["Synchronized"] = QString::fromUtf8("true");
	mPropertyDefault["TrikWaitForSonarDistance"]["Distance"] = tr("0");
	mPropertyDefault["TrikWaitForSonarDistance"]["Port"] = QString::fromUtf8("D1");
	mPropertyDefault["TrikWaitForSonarDistance"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitForTouchSensor"]["Port"] = QString::fromUtf8("A1");
	mPropertyDefault["TrikWaitGamepadButton"]["Button"] = QString::fromUtf8("1");
	mPropertyDefault["TrikWaitGamepadWheel"]["Angle"] = tr("0");
	mPropertyDefault["TrikWaitGamepadWheel"]["Sign"] = QString::fromUtf8("greater");
	mPropertyDefault["TrikWaitPadPress"]["Pad"] = QString::fromUtf8("1");
	mPropertyDefault["TrikWriteToFile"]["File"] = tr("output.txt");
	mPropertyDefault["VariableInit"]["value"] = tr("0");
	mPropertyDefault["VariableInit"]["variable"] = tr("x");
}

void RobotsMetamodelPlugin::initDescriptionMap()
{
	mElementsDescriptionMap["RobotsDiagram"]["ClearScreen"] = tr("Clears everything drawn on the robot`s screen.");
	mElementsDescriptionMap["RobotsDiagram"]["CommentBlock"] = tr("This block can hold text notes that are ignored by generators and interpreters. Use it for improving the diagram readability.");
	mElementsDescriptionMap["RobotsDiagram"]["ControlFlow"] = tr("For creating a link between two elements A and B you can just hover a mouse above A, press the right mouse button and (without releasing it) draw a line to the element B. Alternatively you can just 'pull' a link from a small blue circle next to the element.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3Beep"] = tr("Plays on the robot a sound with the fixed frequency. There are two parameters. The first one is a loudness of the sound, the second means if program should wait for sound completion or go to next block right away.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3ClearEncoder"] = tr("Nullifies tacho limit of the motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3DrawCircle"] = tr("Draws on the robot screen a circle with the given center and radius.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3DrawLine"] = tr("Draws a segment on the robot screen. The parameters specify the ends of the segment.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3DrawPixel"] = tr("Draws one pixel in the specified coordinates on the robot screen.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3DrawRect"] = tr("Draws a rectangle on the robot screen. The parameters specify the coordinates of top-left corner, the width and the height of the rectangle.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3EnginesBackward"] = tr("Enables motors on the given ports in reverse mode with the given power. Ports are specified with A, B or C letters divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the usual mode.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3EnginesForward"] = tr("Enables motors on the given ports with the given power. Ports are specified with A, B or C letters divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the reverse mode.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3EnginesStop"] = tr("Disables motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3Led"] = tr("Sets the color of the LED on the robot`s front panel.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3PlayTone"] = tr("Plays on the robot a sound with the given frequency and duration. This block is similar to the 'Beep' block wuth the only difference that here you can specify sound parameters.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForButton"] = tr("Waits for press of a button on a brick.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForColor"] = tr("Waits till the color sensor on the given port will recognize the given color.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForColorIntensity"] = tr("Waits till the value returned by the color sensor on the given port will be greater or less than the given in the 'Intensity' parameter value (the intensity is specified in percents, 0 to 100).");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForEncoder"] = tr("Waits till the tacho limit of the motor on the given port will reach the value of the 'Tacho Limit' parameter.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForLight"] = tr("Waits till the value returned by the light sensor on the given port will be greater or less than the given in the 'Percents' parameter value (0 to 100).");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForSonarDistance"] = tr("Waits till the value returned by the ultrasonic or infrared range sensor on the given port will be greater or less than the given in the 'Distance' parameter value (the distance is specified in centimeters, 0 to 255).");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForSound"] = tr("Waits till the loudness obtained by the sound sensor on the given port will be greater or less than the given value.");
	mElementsDescriptionMap["RobotsDiagram"]["Ev3WaitForTouchSensor"] = tr("Waits till the touch sensor is pressed. The only parameter is a sensor`s port number (1, 2, 3 or 4).");
	mElementsDescriptionMap["RobotsDiagram"]["FiBlock"] = tr("Unites control flow from different condition branches.");
	mElementsDescriptionMap["RobotsDiagram"]["FinalNode"] = tr("The final node of the program. If the program consists of some parallel execution lines the reachment of this block terminates the corresponding execution line. This block can`t have outgoing links.");
	mElementsDescriptionMap["RobotsDiagram"]["Fork"] = tr("Separates program execution into a number of threads that will be executed concurrently from the programmers`s point of view. For example in such way signal from sensor and some time interval can be waited synchroniously. This block must have at least two outgoing links. 'Guard' property of every link must contain unique thread identifiers, and one of those identifiers must be the same as the identifier of a thread where fork is placed (it must be 'main' if it is the first fork in a program.");
	mElementsDescriptionMap["RobotsDiagram"]["Function"] = tr("Evaluates a value of the given expression. Also new variables can be defined in this block. See the 'Expressions Syntax' chapter in help for more information about 'Function' block syntax.");
	mElementsDescriptionMap["RobotsDiagram"]["GetButtonCode"] = tr("Assigns a given variable a value of pressed button. If no button is pressed at the moment and 'Wait' property is false when variable is set to -1.");
	mElementsDescriptionMap["RobotsDiagram"]["IfBlock"] = tr("Separates program execution in correspondece with the given condition. The 'Condition' parameter value must be some boolean expression that will determine subsequent program execution line. This block must have two outgoing links, at least one of them must have 'Guard' parameter set to 'true' or 'false'. The execution will be proceed trough the link marked with the guard corresponding to 'Condition' parameter of the block.");
	mElementsDescriptionMap["RobotsDiagram"]["InitialNode"] = tr("The entry point of the program execution. Each diagram should have only one such block, it must not have incomming links and it must have only one outgoing link. The interpretation process starts from exactly this block.");
	mElementsDescriptionMap["RobotsDiagram"]["Join"] = tr("Joins a number of threads into one. 'Guard' property of the single outgoing link must contain an identifier of one of threads being joined. The specified thread would wait until the rest of them finish execution, and then proceed in a normal way.");
	mElementsDescriptionMap["RobotsDiagram"]["KillThread"] = tr("Terminates execution of a specified thread.");
	mElementsDescriptionMap["RobotsDiagram"]["Loop"] = tr("This block executes a sequence of blocks for a given number of times. The number of repetitions is specified by the 'Iterations' parameter. This block must have two outgoing links. One of them must be marked with the 'body' guard (that means that the property 'Guard' of the link must be set to 'body' value). Another outgoing link must be unmarked: the program execution will be proceeded through this link when it will go through our 'Loop' block for the given number of times.");
	mElementsDescriptionMap["RobotsDiagram"]["MarkerDown"] = tr("Moves the marker of the 2D model robot down to the floor: the robot will draw its trace on the floor after that. If the marker of another color is already drawing at the moment it will be replaced.");
	mElementsDescriptionMap["RobotsDiagram"]["MarkerUp"] = tr("Lifts the marker of the 2D model robot up: the robot stops drawing its trace on the floor after that.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtBeep"] = tr("Plays on the robot a sound with the fixed frequency. There are two parameters. The first one is a loudness of the sound, the second means if program should wait for sound completion or go to next block right away.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtClearEncoder"] = tr("Nullifies tacho limit of the motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtDrawCircle"] = tr("Draws on the robot screen a circle with the given center and radius.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtDrawLine"] = tr("Draws a segment on the robot screen. The parameters specify the ends of the segment.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtDrawPixel"] = tr("Draws one pixel in the specified coordinates on the robot screen.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtDrawRect"] = tr("Draws a rectangle on the robot screen. The parameters specify the coordinates of top-left corner, the width and the height of the rectangle.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtEnginesBackward"] = tr("Enables motors on the given ports in reverse mode with the given power. Ports are specified with A, B or C letters divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the usual mode.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtEnginesForward"] = tr("Enables motors on the given ports with the given power. Ports are specified with A, B or C letters divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the reverse mode.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtEnginesStop"] = tr("Disables motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtPlayTone"] = tr("Plays on the robot a sound with the given frequency and duration. This block is similar to the 'Beep' block wuth the only difference that here you can specify sound parameters.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForButton"] = tr("Waits for press of a button on a brick.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForColor"] = tr("Waits till the color sensor on the given port will recognize the given color.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForColorIntensity"] = tr("Waits till the value returned by the color sensor on the given port will be greater or less than the given in the 'Intensity' parameter value (the intensity is specified in percents, 0 to 100).");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForEncoder"] = tr("Waits till the tacho limit of the motor on the given port will reach the value of the 'Tacho Limit' parameter.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForLight"] = tr("Waits till the value returned by the light sensor on the given port will be greater or less than the given in the 'Percents' parameter value (0 to 100).");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForSonarDistance"] = tr("Waits till the value returned by the ultrasonic sensor on the given port will be greater or less than the given in the 'Distance' parameter value (the distance is specified in centimeters, 0 to 255).");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForSound"] = tr("Waits till the loudness obtained by the sound sensor on the given port will be greater or less than the given value.");
	mElementsDescriptionMap["RobotsDiagram"]["NxtWaitForTouchSensor"] = tr("Waits till the touch sensor is pressed. The only parameter is a sensor`s port number (1, 2, 3 or 4).");
	mElementsDescriptionMap["RobotsDiagram"]["PrintText"] = tr("Prints a given line in the specified coordinates on the robot`s screen. The value of 'Text' property is interpreted as a plain text unless 'Evaluate' property is set to true, then it will be interpreted as an expression (that may be useful for example when debugging variables values).");
	mElementsDescriptionMap["RobotsDiagram"]["Randomizer"] = tr("Sets a variable value to a random value inside given interval.");
	mElementsDescriptionMap["RobotsDiagram"]["ReceiveMessageThreads"] = tr("Receive a message sent to a thread from which this block is called.");
	mElementsDescriptionMap["RobotsDiagram"]["SendMessageThreads"] = tr("Sends a message to a specified thread.");
	mElementsDescriptionMap["RobotsDiagram"]["Subprogram"] = tr("Subprogram call. Subprograms are used for splitting repetitive program parts into a separate diagram and then calling it from main diagram or other subprograms. When this block is added into a diagram it will be suggested to enter subprogram name. The double click on subprogram call block may open the corresponding subprogram diagram. Moreover user palette will appear containing existing subrpograms, they can be dragged into a diagram like the usual blocks.");
	mElementsDescriptionMap["RobotsDiagram"]["SwitchBlock"] = tr("Selects the program execution branch in correspondence with some expression value. The value of the expression written in 'Expression' property is compared to the values on the outgoing links. If equal value is found then execution will be proceeded by that branch. Else branch without a marker will be selected.");
	mElementsDescriptionMap["RobotsDiagram"]["Timer"] = tr("Waits for a given time in milliseconds.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikAngularServo"] = tr("Manages angular servomotor. Sets up rotation angle on the given port in degrees. Values from 0 to 90 are correspond to a clockwise rotation and values from -90 to 0 correspond to counterclockwise rotation.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDetect"] = tr("Initializes videocamera line or object detector with the color of the object in the middle of the camera`s sight.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDetectorToVariable"] = tr("Stores videocamera line detector`s value into a given variable.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDrawArc"] = tr("Draws the arc defined by the rectangle beginning at (x, y) with the specified width and height, and the given startAngle and spanAngle.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDrawEllipse"] = tr("Draws the ellipse defined by the rectangle beginning at (x, y) with the given width and height.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDrawLine"] = tr("Draws a segment on the robot screen. The parameters specify the ends of the segment.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDrawPixel"] = tr("Draws one pixel in the specified coordinates on the robot screen.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikDrawRect"] = tr("Draws a rectangle on the robot screen. The parameters specify the coordinates of top-left corner, the width and the height of the rectangle.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikInitCamera"] = tr("Enables line or object or color detector by videocamera and draws videostream on the robot`s screen.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikInitVideoStreaming"] = tr("Enables video stream on robot. Video then can be watched on TRIK gamepad or in browser using URL http://ROBOT.IP.ADDRESS:8080/?action=stream");
	mElementsDescriptionMap["RobotsDiagram"]["TrikLed"] = tr("Sets the color of the LED on the robot`s front panel.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikPlayTone"] = tr("Plays on the robot a sound file (.wav or .mp3) previously uploaded to it.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikRemoveFile"] = tr("Removes a file. File path may be absolute or relative to a directory containing TRIK Studio executable.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSadSmile"] = tr("Draws a sad smile on the robot`s screen :(");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSay"] = tr("Synthesizes the given phrase and plays it on the robot. If 'Evaluate' property is set to true then the value of 'Text' property is interpreted as a formula, otherwise it is interpreted as plain string.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSendMessage"] = tr("Sends message to another robot.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSetBackground"] = tr("Sets the background color of the current picture on the robot`s screen.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSetPainterColor"] = tr("Sets the painter color.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSetPainterWidth"] = tr("Sets the painter width.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSmile"] = tr("Draws a smile on the robot`s screen :)");
	mElementsDescriptionMap["RobotsDiagram"]["TrikSystem"] = tr("Invokes bash script-style command on the robot. If 'Evaluate' property is set to true then the value of 'Command' property is interpreted as a formula, otherwise it is interpreted as plain string.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV62AngularServo"] = tr("Manages angular servomotor. Sets up rotation angle on the given port in degrees. Values from 0 to 90 are correspond to a clockwise rotation and values from -90 to 0 correspond to counterclockwise rotation.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV62ClearEncoder"] = tr("Nullifies tacho limit of the motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV62WaitForEncoder"] = tr("Waits till the tacho limit of the motor on the given port will reach the value of the 'Tacho Limit' parameter.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV6ClearEncoder"] = tr("Nullifies tacho limit of the motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV6EnginesBackward"] = tr("Enables motors on the given ports in reverse mode with the given power. Ports are specified in accordance with TRIK controller notations (for example M1, E2) and divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the usual mode.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV6EnginesForward"] = tr("Enables motors on the given ports with the given power. Ports are specified in accordance with TRIK controller notations (for example M1, E2) and divided by commas. The power is specified in percents with the number from -100 to 100, if negative number is specified then the motor is enabled in the reverse mode.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikV6EnginesStop"] = tr("Disables motors on the given ports.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForButton"] = tr("Waits for press of a button on a brick.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForEncoder"] = tr("Waits till the tacho limit of the motor on the given port will reach the value of the 'Tacho Limit' parameter.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForGyroscope"] = tr("Waits till the value returned by the gyroscope on the given port will be greater or less than the given in the 'Degrees/sec' parameter value.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForIRDistance"] = tr("Waits till the value returned by the infrared sensor on the given port will be greater or less than the given in the 'Distance' parameter value. By default on ports A1 and A2 the distance is specified in centimeters (from 0 to 100). It is not recommended to plug IR sensor into other ports because its raw value will be processed by software on robot with expectation of another sensor type.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForLight"] = tr("Waits till the value returned by the light sensor on the given port will be greater or less than the given in the 'Percents' parameter value (0 to 100).");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForMessage"] = tr("Stores a message from another robot into a given variable. When no incoming messages are present at the moment, a robot will wait for incoming message if 'Synchronized' property is true, and empty string will be assigned to a variable otherwise.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForMotion"] = tr("Waits till the motion sensor on the given port is triggered.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForSonarDistance"] = tr("Waits till the value returned by the ultrasonic sensor on the given port will be greater or less than the given in the 'Distance' parameter value (the distance is specified in centimeters, from 0 to 300).");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitForTouchSensor"] = tr("Waits till the touch sensor is pressed. The only parameter is a sensor`s port number (1, 2, 3 or 4).");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitGamepadButton"] = tr("Waits for Android gamepad 'magic button' press. Buttons are identified by numbers from 1 to 5 (from left to right)");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitGamepadConnect"] = tr("Waits until Android gamepad is connected.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitGamepadDisconnect"] = tr("Waits until Android gamepad is disconnected.");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitGamepadWheel"] = tr("Waits till the value returned by gamepad wheel be greater or less than the given in the 'Angle' parameter value. Angle is measured from -100 (max left) to 100 (max right).");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWaitPadPress"] = tr("Waits for Android gamepad pad press (left pad has number 1, right pad - number 2).");
	mElementsDescriptionMap["RobotsDiagram"]["TrikWriteToFile"] = tr("Writes a given message to a file. File path may be absolute or relative to a directory containing TRIK Studio executable.");
	mElementsDescriptionMap["RobotsDiagram"]["VariableInit"] = tr("Assigns a given value to a given variable.");
}

void RobotsMetamodelPlugin::initPortTypes()
{
	mPortTypes = QMap<QString, QStringList>({
		{"AbstractNode", {"NonTyped", }},
		{"ClearScreen", {"NonTyped", }},
		{"CommentBlock", {"NonTyped", }},
		{"ControlFlow", {}},
		{"EngineCommand", {"NonTyped", }},
		{"EngineMovementCommand", {"NonTyped", }},
		{"Ev3Beep", {"NonTyped", }},
		{"Ev3ClearEncoder", {"NonTyped", }},
		{"Ev3DrawCircle", {"NonTyped", }},
		{"Ev3DrawLine", {"NonTyped", }},
		{"Ev3DrawPixel", {"NonTyped", }},
		{"Ev3DrawRect", {"NonTyped", }},
		{"Ev3EngineMovementCommand", {"NonTyped", }},
		{"Ev3EnginesBackward", {"NonTyped", }},
		{"Ev3EnginesForward", {"NonTyped", }},
		{"Ev3EnginesStop", {"NonTyped", }},
		{"Ev3Led", {"NonTyped", }},
		{"Ev3PlayTone", {"NonTyped", }},
		{"Ev3SensorBlock", {"NonTyped", }},
		{"Ev3WaitForButton", {"NonTyped", }},
		{"Ev3WaitForColor", {"NonTyped", }},
		{"Ev3WaitForColorIntensity", {"NonTyped", }},
		{"Ev3WaitForEncoder", {"NonTyped", }},
		{"Ev3WaitForLight", {"NonTyped", }},
		{"Ev3WaitForSonarDistance", {"NonTyped", }},
		{"Ev3WaitForSound", {"NonTyped", }},
		{"Ev3WaitForTouchSensor", {"NonTyped", }},
		{"FiBlock", {"NonTyped", }},
		{"FinalNode", {"NonTyped", }},
		{"Fork", {"NonTyped", }},
		{"Function", {"NonTyped", }},
		{"GetButtonCode", {"NonTyped", }},
		{"IfBlock", {"NonTyped", }},
		{"InitialNode", {"NonTyped", }},
		{"Join", {"NonTyped", }},
		{"KillThread", {"NonTyped", }},
		{"Loop", {"NonTyped", }},
		{"MarkerDown", {"NonTyped", }},
		{"MarkerUp", {"NonTyped", }},
		{"NxtBeep", {"NonTyped", }},
		{"NxtClearEncoder", {"NonTyped", }},
		{"NxtDrawCircle", {"NonTyped", }},
		{"NxtDrawLine", {"NonTyped", }},
		{"NxtDrawPixel", {"NonTyped", }},
		{"NxtDrawRect", {"NonTyped", }},
		{"NxtEngineMovementCommand", {"NonTyped", }},
		{"NxtEnginesBackward", {"NonTyped", }},
		{"NxtEnginesForward", {"NonTyped", }},
		{"NxtEnginesStop", {"NonTyped", }},
		{"NxtPlayTone", {"NonTyped", }},
		{"NxtSensorBlock", {"NonTyped", }},
		{"NxtWaitForButton", {"NonTyped", }},
		{"NxtWaitForColor", {"NonTyped", }},
		{"NxtWaitForColorIntensity", {"NonTyped", }},
		{"NxtWaitForEncoder", {"NonTyped", }},
		{"NxtWaitForLight", {"NonTyped", }},
		{"NxtWaitForSonarDistance", {"NonTyped", }},
		{"NxtWaitForSound", {"NonTyped", }},
		{"NxtWaitForTouchSensor", {"NonTyped", }},
		{"PrintText", {"NonTyped", }},
		{"Randomizer", {"NonTyped", }},
		{"ReceiveMessageThreads", {"NonTyped", }},
		{"RobotsDiagramNode", {"NonTyped", }},
		{"SendMessageThreads", {"NonTyped", }},
		{"Subprogram", {"NonTyped", }},
		{"SubprogramDiagram", {"NonTyped", }},
		{"SwitchBlock", {"NonTyped", }},
		{"Timer", {"NonTyped", }},
		{"TrikAnalogSensorBlock", {"NonTyped", }},
		{"TrikAngularServo", {"NonTyped", }},
		{"TrikDetect", {"NonTyped", }},
		{"TrikDetectorToVariable", {"NonTyped", }},
		{"TrikDigitalSensorBlock", {"NonTyped", }},
		{"TrikDrawArc", {"NonTyped", }},
		{"TrikDrawEllipse", {"NonTyped", }},
		{"TrikDrawLine", {"NonTyped", }},
		{"TrikDrawPixel", {"NonTyped", }},
		{"TrikDrawRect", {"NonTyped", }},
		{"TrikInitCamera", {"NonTyped", }},
		{"TrikInitVideoStreaming", {"NonTyped", }},
		{"TrikLed", {"NonTyped", }},
		{"TrikPlayTone", {"NonTyped", }},
		{"TrikRemoveFile", {"NonTyped", }},
		{"TrikSadSmile", {"NonTyped", }},
		{"TrikSay", {"NonTyped", }},
		{"TrikSendMessage", {"NonTyped", }},
		{"TrikSensorBlock", {"NonTyped", }},
		{"TrikSetBackground", {"NonTyped", }},
		{"TrikSetPainterColor", {"NonTyped", }},
		{"TrikSetPainterWidth", {"NonTyped", }},
		{"TrikSmile", {"NonTyped", }},
		{"TrikSystem", {"NonTyped", }},
		{"TrikV62AngularServo", {"NonTyped", }},
		{"TrikV62ClearEncoder", {"NonTyped", }},
		{"TrikV62WaitForEncoder", {"NonTyped", }},
		{"TrikV6ClearEncoder", {"NonTyped", }},
		{"TrikV6EngineMovementCommand", {"NonTyped", }},
		{"TrikV6EnginesBackward", {"NonTyped", }},
		{"TrikV6EnginesForward", {"NonTyped", }},
		{"TrikV6EnginesStop", {"NonTyped", }},
		{"TrikWaitForAccelerometer", {"NonTyped", }},
		{"TrikWaitForButton", {"NonTyped", }},
		{"TrikWaitForEncoder", {"NonTyped", }},
		{"TrikWaitForGyroscope", {"NonTyped", }},
		{"TrikWaitForIRDistance", {"NonTyped", }},
		{"TrikWaitForLight", {"NonTyped", }},
		{"TrikWaitForMessage", {"NonTyped", }},
		{"TrikWaitForMotion", {"NonTyped", }},
		{"TrikWaitForSonarDistance", {"NonTyped", }},
		{"TrikWaitForTouchSensor", {"NonTyped", }},
		{"TrikWaitGamepadButton", {"NonTyped", }},
		{"TrikWaitGamepadConnect", {"NonTyped", }},
		{"TrikWaitGamepadDisconnect", {"NonTyped", }},
		{"TrikWaitGamepadWheel", {"NonTyped", }},
		{"TrikWaitPadPress", {"NonTyped", }},
		{"TrikWriteToFile", {"NonTyped", }},
		{"VariableInit", {"NonTyped", }},
	});
}

void RobotsMetamodelPlugin::initParentsMap()
{
	mParentsMap["RobotsDiagram"]["ClearScreen"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["CommentBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["EngineCommand"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["EngineMovementCommand"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["Ev3Beep"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3ClearEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3DrawCircle"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3DrawLine"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3DrawPixel"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3DrawRect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3EngineMovementCommand"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["Ev3EnginesBackward"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["Ev3EnginesForward"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["Ev3EnginesStop"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["Ev3Led"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3PlayTone"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3SensorBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForButton"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForColor"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForColorIntensity"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForLight"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForSonarDistance"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForSound"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["Ev3WaitForTouchSensor"]
		<< qMakePair(QString("RobotsDiagram"), QString("Ev3SensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["FiBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["FinalNode"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Fork"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Function"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["GetButtonCode"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["IfBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["InitialNode"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Join"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["KillThread"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Loop"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["MarkerDown"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["MarkerUp"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtBeep"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtClearEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtDrawCircle"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtDrawLine"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtDrawPixel"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtDrawRect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtEngineMovementCommand"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["NxtEnginesBackward"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtEngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["NxtEnginesForward"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtEngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["NxtEnginesStop"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["NxtPlayTone"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtSensorBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForButton"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForColor"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForColorIntensity"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForLight"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForSonarDistance"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForSound"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["NxtWaitForTouchSensor"]
		<< qMakePair(QString("RobotsDiagram"), QString("NxtSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["PrintText"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Randomizer"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["ReceiveMessageThreads"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["SendMessageThreads"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Subprogram"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["SwitchBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["Timer"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikAnalogSensorBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikAngularServo"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikDetect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDetectorToVariable"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDigitalSensorBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikDrawArc"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDrawEllipse"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDrawLine"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDrawPixel"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikDrawRect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikInitCamera"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikInitVideoStreaming"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikLed"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikPlayTone"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikRemoveFile"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSadSmile"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSay"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSendMessage"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSensorBlock"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSetBackground"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSetPainterColor"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSetPainterWidth"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSmile"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikSystem"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikV62AngularServo"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikV62ClearEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikV62WaitForEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikV6ClearEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikV6EngineMovementCommand"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikV6EnginesBackward"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikV6EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikV6EnginesForward"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikV6EngineMovementCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikV6EnginesStop"]
		<< qMakePair(QString("RobotsDiagram"), QString("EngineCommand"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForAccelerometer"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForButton"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForEncoder"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForGyroscope"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForIRDistance"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikAnalogSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForLight"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikAnalogSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForMessage"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForMotion"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForSonarDistance"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikDigitalSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitForTouchSensor"]
		<< qMakePair(QString("RobotsDiagram"), QString("TrikAnalogSensorBlock"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitGamepadButton"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitGamepadConnect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitGamepadDisconnect"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitGamepadWheel"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWaitPadPress"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["TrikWriteToFile"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
	mParentsMap["RobotsDiagram"]["VariableInit"]
		<< qMakePair(QString("RobotsDiagram"), QString("AbstractNode"))
	;
}

void RobotsMetamodelPlugin::initShallPaletteBeSortedMap()
{
	mShallPaletteBeSortedMap[QString::fromUtf8("RobotsDiagram")] = false;
}

void RobotsMetamodelPlugin::initExplosionsMap()
{
	mExplosionsMap["RobotsDiagram"]["Subprogram"] << ExplosionData("RobotsDiagram", "SubprogramDiagramGroup", true, true);
}

QStringList RobotsMetamodelPlugin::diagrams() const
{
	return mDiagramNameMap.keys();
}

QStringList RobotsMetamodelPlugin::diagramPaletteGroups(const QString &diagram) const
{
	QStringList result;
	for (QPair<QString, QStringList> const &group : mPaletteGroupsMap[diagram]) {
		result << group.first;
	}

	return result;
}

QStringList RobotsMetamodelPlugin::diagramPaletteGroupList(const QString &diagram, const QString &group) const
{
	for (QPair<QString, QStringList> const &ourGroup : mPaletteGroupsMap[diagram]) {
		if (ourGroup.first == group) {
			return ourGroup.second;
		}
	}

	return QStringList();
}

QString RobotsMetamodelPlugin::diagramPaletteGroupDescription(const QString &diagram, const QString &group) const
{
	return mPaletteGroupsDescriptionMap[diagram][group];
}

bool RobotsMetamodelPlugin::shallPaletteBeSorted(const QString &diagram) const
{
	return mShallPaletteBeSortedMap[diagram];
}

QStringList RobotsMetamodelPlugin::elements(const QString &diagram) const
{
	return mElementsNameMap[diagram].keys();
}

QStringList RobotsMetamodelPlugin::getPropertiesWithDefaultValues(const QString &element) const
{
	return mPropertyDefault[element].keys();
}

QString RobotsMetamodelPlugin::editorName() const
{
	 return "RobotsMetamodel Editor";
}

QString RobotsMetamodelPlugin::diagramName(const QString &diagram) const
{
	return mDiagramNameMap[diagram];
}

QString RobotsMetamodelPlugin::diagramNodeName(const QString &diagram) const
{
	return mDiagramNodeNameMap[diagram];
}

QString RobotsMetamodelPlugin::elementName(const QString &diagram, const QString &element) const
{
	return mElementsNameMap[diagram][element];
}

QString RobotsMetamodelPlugin::elementDescription(const QString &diagram, const QString &element)const
{
	return mElementsDescriptionMap[diagram][element];
}

QString RobotsMetamodelPlugin::propertyDescription(const QString &diagram, const QString &element, const QString &property) const
{
	return mPropertiesDescriptionMap[diagram][element][property];
}

QString RobotsMetamodelPlugin::propertyDisplayedName(const QString &diagram, const QString &element, const QString &property) const
{
	return mPropertiesDisplayedNamesMap[diagram][element][property];
}

QString RobotsMetamodelPlugin::elementMouseGesture(const QString &diagram, const QString &element) const
{
	return mElementMouseGesturesMap[diagram][element];
}

QList<qReal::EditorInterface::ExplosionData>RobotsMetamodelPlugin::explosions(const QString &diagram, const QString &element) const 
{
	return mExplosionsMap[diagram][element];
}

qReal::ElementImpl* RobotsMetamodelPlugin::getGraphicalObject(const QString &/*diagram*/, const QString &element) const
{
	if (element == "AbstractNode") {
		return new AbstractNode();
	}
	else if (element == "ClearScreen") {
		return new ClearScreen();
	}
	else if (element == "CommentBlock") {
		return new CommentBlock();
	}
	else if (element == "ControlFlow") {
		return new ControlFlow();
	}
	else if (element == "Ev3Beep") {
		return new Ev3Beep();
	}
	else if (element == "Ev3ClearEncoder") {
		return new Ev3ClearEncoder();
	}
	else if (element == "Ev3DrawCircle") {
		return new Ev3DrawCircle();
	}
	else if (element == "Ev3DrawLine") {
		return new Ev3DrawLine();
	}
	else if (element == "Ev3DrawPixel") {
		return new Ev3DrawPixel();
	}
	else if (element == "Ev3DrawRect") {
		return new Ev3DrawRect();
	}
	else if (element == "Ev3EnginesBackward") {
		return new Ev3EnginesBackward();
	}
	else if (element == "Ev3EnginesForward") {
		return new Ev3EnginesForward();
	}
	else if (element == "Ev3EnginesStop") {
		return new Ev3EnginesStop();
	}
	else if (element == "Ev3Led") {
		return new Ev3Led();
	}
	else if (element == "Ev3PlayTone") {
		return new Ev3PlayTone();
	}
	else if (element == "Ev3WaitForButton") {
		return new Ev3WaitForButton();
	}
	else if (element == "Ev3WaitForColor") {
		return new Ev3WaitForColor();
	}
	else if (element == "Ev3WaitForColorIntensity") {
		return new Ev3WaitForColorIntensity();
	}
	else if (element == "Ev3WaitForEncoder") {
		return new Ev3WaitForEncoder();
	}
	else if (element == "Ev3WaitForLight") {
		return new Ev3WaitForLight();
	}
	else if (element == "Ev3WaitForSonarDistance") {
		return new Ev3WaitForSonarDistance();
	}
	else if (element == "Ev3WaitForSound") {
		return new Ev3WaitForSound();
	}
	else if (element == "Ev3WaitForTouchSensor") {
		return new Ev3WaitForTouchSensor();
	}
	else if (element == "FiBlock") {
		return new FiBlock();
	}
	else if (element == "FinalNode") {
		return new FinalNode();
	}
	else if (element == "Fork") {
		return new Fork();
	}
	else if (element == "Function") {
		return new Function();
	}
	else if (element == "GetButtonCode") {
		return new GetButtonCode();
	}
	else if (element == "IfBlock") {
		return new IfBlock();
	}
	else if (element == "InitialNode") {
		return new InitialNode();
	}
	else if (element == "Join") {
		return new Join();
	}
	else if (element == "KillThread") {
		return new KillThread();
	}
	else if (element == "Loop") {
		return new Loop();
	}
	else if (element == "MarkerDown") {
		return new MarkerDown();
	}
	else if (element == "MarkerUp") {
		return new MarkerUp();
	}
	else if (element == "NxtBeep") {
		return new NxtBeep();
	}
	else if (element == "NxtClearEncoder") {
		return new NxtClearEncoder();
	}
	else if (element == "NxtDrawCircle") {
		return new NxtDrawCircle();
	}
	else if (element == "NxtDrawLine") {
		return new NxtDrawLine();
	}
	else if (element == "NxtDrawPixel") {
		return new NxtDrawPixel();
	}
	else if (element == "NxtDrawRect") {
		return new NxtDrawRect();
	}
	else if (element == "NxtEnginesBackward") {
		return new NxtEnginesBackward();
	}
	else if (element == "NxtEnginesForward") {
		return new NxtEnginesForward();
	}
	else if (element == "NxtEnginesStop") {
		return new NxtEnginesStop();
	}
	else if (element == "NxtPlayTone") {
		return new NxtPlayTone();
	}
	else if (element == "NxtWaitForButton") {
		return new NxtWaitForButton();
	}
	else if (element == "NxtWaitForColor") {
		return new NxtWaitForColor();
	}
	else if (element == "NxtWaitForColorIntensity") {
		return new NxtWaitForColorIntensity();
	}
	else if (element == "NxtWaitForEncoder") {
		return new NxtWaitForEncoder();
	}
	else if (element == "NxtWaitForLight") {
		return new NxtWaitForLight();
	}
	else if (element == "NxtWaitForSonarDistance") {
		return new NxtWaitForSonarDistance();
	}
	else if (element == "NxtWaitForSound") {
		return new NxtWaitForSound();
	}
	else if (element == "NxtWaitForTouchSensor") {
		return new NxtWaitForTouchSensor();
	}
	else if (element == "PrintText") {
		return new PrintText();
	}
	else if (element == "Randomizer") {
		return new Randomizer();
	}
	else if (element == "ReceiveMessageThreads") {
		return new ReceiveMessageThreads();
	}
	else if (element == "RobotsDiagramNode") {
		return new RobotsDiagramNode();
	}
	else if (element == "SendMessageThreads") {
		return new SendMessageThreads();
	}
	else if (element == "Subprogram") {
		return new Subprogram();
	}
	else if (element == "SubprogramDiagram") {
		return new SubprogramDiagram();
	}
	else if (element == "SwitchBlock") {
		return new SwitchBlock();
	}
	else if (element == "Timer") {
		return new Timer();
	}
	else if (element == "TrikAngularServo") {
		return new TrikAngularServo();
	}
	else if (element == "TrikDetect") {
		return new TrikDetect();
	}
	else if (element == "TrikDetectorToVariable") {
		return new TrikDetectorToVariable();
	}
	else if (element == "TrikDrawArc") {
		return new TrikDrawArc();
	}
	else if (element == "TrikDrawEllipse") {
		return new TrikDrawEllipse();
	}
	else if (element == "TrikDrawLine") {
		return new TrikDrawLine();
	}
	else if (element == "TrikDrawPixel") {
		return new TrikDrawPixel();
	}
	else if (element == "TrikDrawRect") {
		return new TrikDrawRect();
	}
	else if (element == "TrikInitCamera") {
		return new TrikInitCamera();
	}
	else if (element == "TrikInitVideoStreaming") {
		return new TrikInitVideoStreaming();
	}
	else if (element == "TrikLed") {
		return new TrikLed();
	}
	else if (element == "TrikPlayTone") {
		return new TrikPlayTone();
	}
	else if (element == "TrikRemoveFile") {
		return new TrikRemoveFile();
	}
	else if (element == "TrikSadSmile") {
		return new TrikSadSmile();
	}
	else if (element == "TrikSay") {
		return new TrikSay();
	}
	else if (element == "TrikSendMessage") {
		return new TrikSendMessage();
	}
	else if (element == "TrikSetBackground") {
		return new TrikSetBackground();
	}
	else if (element == "TrikSetPainterColor") {
		return new TrikSetPainterColor();
	}
	else if (element == "TrikSetPainterWidth") {
		return new TrikSetPainterWidth();
	}
	else if (element == "TrikSmile") {
		return new TrikSmile();
	}
	else if (element == "TrikSystem") {
		return new TrikSystem();
	}
	else if (element == "TrikV62AngularServo") {
		return new TrikV62AngularServo();
	}
	else if (element == "TrikV62ClearEncoder") {
		return new TrikV62ClearEncoder();
	}
	else if (element == "TrikV62WaitForEncoder") {
		return new TrikV62WaitForEncoder();
	}
	else if (element == "TrikV6ClearEncoder") {
		return new TrikV6ClearEncoder();
	}
	else if (element == "TrikV6EnginesBackward") {
		return new TrikV6EnginesBackward();
	}
	else if (element == "TrikV6EnginesForward") {
		return new TrikV6EnginesForward();
	}
	else if (element == "TrikV6EnginesStop") {
		return new TrikV6EnginesStop();
	}
	else if (element == "TrikWaitForAccelerometer") {
		return new TrikWaitForAccelerometer();
	}
	else if (element == "TrikWaitForButton") {
		return new TrikWaitForButton();
	}
	else if (element == "TrikWaitForEncoder") {
		return new TrikWaitForEncoder();
	}
	else if (element == "TrikWaitForGyroscope") {
		return new TrikWaitForGyroscope();
	}
	else if (element == "TrikWaitForIRDistance") {
		return new TrikWaitForIRDistance();
	}
	else if (element == "TrikWaitForLight") {
		return new TrikWaitForLight();
	}
	else if (element == "TrikWaitForMessage") {
		return new TrikWaitForMessage();
	}
	else if (element == "TrikWaitForMotion") {
		return new TrikWaitForMotion();
	}
	else if (element == "TrikWaitForSonarDistance") {
		return new TrikWaitForSonarDistance();
	}
	else if (element == "TrikWaitForTouchSensor") {
		return new TrikWaitForTouchSensor();
	}
	else if (element == "TrikWaitGamepadButton") {
		return new TrikWaitGamepadButton();
	}
	else if (element == "TrikWaitGamepadConnect") {
		return new TrikWaitGamepadConnect();
	}
	else if (element == "TrikWaitGamepadDisconnect") {
		return new TrikWaitGamepadDisconnect();
	}
	else if (element == "TrikWaitGamepadWheel") {
		return new TrikWaitGamepadWheel();
	}
	else if (element == "TrikWaitPadPress") {
		return new TrikWaitPadPress();
	}
	else if (element == "TrikWriteToFile") {
		return new TrikWriteToFile();
	}
	else if (element == "VariableInit") {
		return new VariableInit();
	}
	else {
		Q_ASSERT(!"Request for creation of an element with unknown name");
		return nullptr;
	}
}

bool RobotsMetamodelPlugin::isParentOf(const QString &parentDiagram, const QString &parentElement, const QString &childDiagram, const QString &childElement) const
{
	if (childDiagram == parentDiagram && childElement == parentElement)
		return true;
	if (mParentsMap[childDiagram][childElement].contains(qMakePair(parentDiagram, parentElement)))
		return true;
	typedef QPair<QString, QString> StringPair;
	for (const StringPair &pair : mParentsMap[childDiagram][childElement])
		if (isParentOf(parentDiagram, parentElement, pair.first, pair.second))
			return true;
	return false;
}
QList<QPair<QString, QString> > RobotsMetamodelPlugin::getParentsOf(const QString &diagram, const QString &element) const
{
	return mParentsMap[diagram][element];
}
QStringList RobotsMetamodelPlugin::getPropertyNames(const QString &/*diagram*/, const QString &element) const
{
	QStringList result;
	if (element == "AbstractNode") {
	}
	else if (element == "ClearScreen") {
		result  << "Redraw";
	}
	else if (element == "CommentBlock") {
		result  << "Comment";
	}
	else if (element == "ControlFlow") {
		result  << "Guard";
	}
	else if (element == "Ev3Beep") {
		result  << "Volume" << "WaitForCompletion";
	}
	else if (element == "Ev3ClearEncoder") {
		result  << "Ports";
	}
	else if (element == "Ev3DrawCircle") {
		result  << "CircleRadius" << "Filled" << "Redraw" << "XCoordinateCircle" << "YCoordinateCircle"
		;
	}
	else if (element == "Ev3DrawLine") {
		result  << "Redraw" << "X1CoordinateLine" << "X2CoordinateLine" << "Y1CoordinateLine" << "Y2CoordinateLine"
		;
	}
	else if (element == "Ev3DrawPixel") {
		result  << "Redraw" << "XCoordinatePix" << "YCoordinatePix";
	}
	else if (element == "Ev3DrawRect") {
		result  << "Filled" << "HeightRect" << "Redraw" << "WidthRect" << "XCoordinateRect" << "YCoordinateRect"
		;
	}
	else if (element == "Ev3EnginesBackward") {
		result  << "Mode" << "Ports" << "Power";
	}
	else if (element == "Ev3EnginesForward") {
		result  << "Mode" << "Ports" << "Power";
	}
	else if (element == "Ev3EnginesStop") {
		result  << "Ports";
	}
	else if (element == "Ev3Led") {
		result  << "Color";
	}
	else if (element == "Ev3PlayTone") {
		result  << "Duration" << "Frequency" << "Volume" << "WaitForCompletion";
	}
	else if (element == "Ev3WaitForButton") {
		result  << "Button";
	}
	else if (element == "Ev3WaitForColor") {
		result  << "Color" << "Port";
	}
	else if (element == "Ev3WaitForColorIntensity") {
		result  << "Intensity" << "Port" << "Sign";
	}
	else if (element == "Ev3WaitForEncoder") {
		result  << "Port" << "Sign" << "TachoLimit";
	}
	else if (element == "Ev3WaitForLight") {
		result  << "Percents" << "Port" << "Sign";
	}
	else if (element == "Ev3WaitForSonarDistance") {
		result  << "Distance" << "Port" << "Sign";
	}
	else if (element == "Ev3WaitForSound") {
		result  << "Port" << "Sign" << "Volume";
	}
	else if (element == "Ev3WaitForTouchSensor") {
		result  << "Port";
	}
	else if (element == "FiBlock") {
	}
	else if (element == "FinalNode") {
	}
	else if (element == "Fork") {
	}
	else if (element == "Function") {
		result  << "Body";
	}
	else if (element == "GetButtonCode") {
		result  << "Variable" << "Wait";
	}
	else if (element == "IfBlock") {
		result  << "Condition";
	}
	else if (element == "InitialNode") {
	}
	else if (element == "Join") {
	}
	else if (element == "KillThread") {
		result  << "Thread";
	}
	else if (element == "Loop") {
		result  << "Iterations";
	}
	else if (element == "MarkerDown") {
		result  << "Color";
	}
	else if (element == "MarkerUp") {
	}
	else if (element == "NxtBeep") {
		result  << "Volume" << "WaitForCompletion";
	}
	else if (element == "NxtClearEncoder") {
		result  << "Ports";
	}
	else if (element == "NxtDrawCircle") {
		result  << "CircleRadius" << "Redraw" << "XCoordinateCircle" << "YCoordinateCircle";
	}
	else if (element == "NxtDrawLine") {
		result  << "Redraw" << "X1CoordinateLine" << "X2CoordinateLine" << "Y1CoordinateLine" << "Y2CoordinateLine"
		;
	}
	else if (element == "NxtDrawPixel") {
		result  << "Redraw" << "XCoordinatePix" << "YCoordinatePix";
	}
	else if (element == "NxtDrawRect") {
		result  << "HeightRect" << "Redraw" << "WidthRect" << "XCoordinateRect" << "YCoordinateRect"
		;
	}
	else if (element == "NxtEnginesBackward") {
		result  << "Mode" << "Ports" << "Power";
	}
	else if (element == "NxtEnginesForward") {
		result  << "Mode" << "Ports" << "Power";
	}
	else if (element == "NxtEnginesStop") {
		result  << "Ports";
	}
	else if (element == "NxtPlayTone") {
		result  << "Duration" << "Frequency" << "Volume" << "WaitForCompletion";
	}
	else if (element == "NxtWaitForButton") {
		result  << "Button";
	}
	else if (element == "NxtWaitForColor") {
		result  << "Color" << "Port";
	}
	else if (element == "NxtWaitForColorIntensity") {
		result  << "Intensity" << "Port" << "Sign";
	}
	else if (element == "NxtWaitForEncoder") {
		result  << "Port" << "Sign" << "TachoLimit";
	}
	else if (element == "NxtWaitForLight") {
		result  << "Percents" << "Port" << "Sign";
	}
	else if (element == "NxtWaitForSonarDistance") {
		result  << "Distance" << "Port" << "Sign";
	}
	else if (element == "NxtWaitForSound") {
		result  << "Port" << "Sign" << "Volume";
	}
	else if (element == "NxtWaitForTouchSensor") {
		result  << "Port";
	}
	else if (element == "PrintText") {
		result  << "Evaluate" << "PrintText" << "Redraw" << "XCoordinateText" << "YCoordinateText"
		;
	}
	else if (element == "Randomizer") {
		result  << "LowerBound" << "UpperBound" << "Variable";
	}
	else if (element == "ReceiveMessageThreads") {
		result  << "Synchronized" << "Variable";
	}
	else if (element == "RobotsDiagramNode") {
		result  << "devicesConfiguration";
	}
	else if (element == "SendMessageThreads") {
		result  << "Message" << "Thread";
	}
	else if (element == "Subprogram") {
	}
	else if (element == "SubprogramDiagram") {
	}
	else if (element == "SwitchBlock") {
		result  << "Expression";
	}
	else if (element == "Timer") {
		result  << "Delay";
	}
	else if (element == "TrikAngularServo") {
		result  << "Ports" << "Power";
	}
	else if (element == "TrikDetect") {
		result  << "Mode";
	}
	else if (element == "TrikDetectorToVariable") {
		result  << "Variable";
	}
	else if (element == "TrikDrawArc") {
		result  << "HeightArc" << "Redraw" << "SpanAngle" << "StartAngle" << "WidthArc" << "XCoordinateArc"
		 << "YCoordinateArc";
	}
	else if (element == "TrikDrawEllipse") {
		result  << "HeightEllipse" << "Redraw" << "WidthEllipse" << "XCoordinateEllipse" << "YCoordinateEllipse"
		;
	}
	else if (element == "TrikDrawLine") {
		result  << "Redraw" << "X1CoordinateLine" << "X2CoordinateLine" << "Y1CoordinateLine" << "Y2CoordinateLine"
		;
	}
	else if (element == "TrikDrawPixel") {
		result  << "Redraw" << "XCoordinatePix" << "YCoordinatePix";
	}
	else if (element == "TrikDrawRect") {
		result  << "HeightRect" << "Redraw" << "WidthRect" << "XCoordinateRect" << "YCoordinateRect"
		;
	}
	else if (element == "TrikInitCamera") {
		result  << "Mode";
	}
	else if (element == "TrikInitVideoStreaming") {
	}
	else if (element == "TrikLed") {
		result  << "Color";
	}
	else if (element == "TrikPlayTone") {
		result  << "FileName";
	}
	else if (element == "TrikRemoveFile") {
		result  << "File";
	}
	else if (element == "TrikSadSmile") {
	}
	else if (element == "TrikSay") {
		result  << "Evaluate" << "Text";
	}
	else if (element == "TrikSendMessage") {
		result  << "HullNumber" << "Message";
	}
	else if (element == "TrikSetBackground") {
		result  << "Color" << "Redraw";
	}
	else if (element == "TrikSetPainterColor") {
		result  << "Color";
	}
	else if (element == "TrikSetPainterWidth") {
		result  << "Width";
	}
	else if (element == "TrikSmile") {
	}
	else if (element == "TrikSystem") {
		result  << "Code" << "Command" << "Evaluate";
	}
	else if (element == "TrikV62AngularServo") {
		result  << "Ports" << "Power";
	}
	else if (element == "TrikV62ClearEncoder") {
		result  << "Ports";
	}
	else if (element == "TrikV62WaitForEncoder") {
		result  << "Port" << "Sign" << "TachoLimit";
	}
	else if (element == "TrikV6ClearEncoder") {
		result  << "Ports";
	}
	else if (element == "TrikV6EnginesBackward") {
		result  << "Ports" << "Power";
	}
	else if (element == "TrikV6EnginesForward") {
		result  << "Ports" << "Power";
	}
	else if (element == "TrikV6EnginesStop") {
		result  << "Ports";
	}
	else if (element == "TrikWaitForAccelerometer") {
		result  << "Acceleration" << "AccelerationAxis" << "Sign";
	}
	else if (element == "TrikWaitForButton") {
		result  << "Button";
	}
	else if (element == "TrikWaitForEncoder") {
		result  << "Port" << "Sign" << "TachoLimit";
	}
	else if (element == "TrikWaitForGyroscope") {
		result  << "Degrees" << "Sign";
	}
	else if (element == "TrikWaitForIRDistance") {
		result  << "Distance" << "Port" << "Sign";
	}
	else if (element == "TrikWaitForLight") {
		result  << "Percents" << "Port" << "Sign";
	}
	else if (element == "TrikWaitForMessage") {
		result  << "Synchronized" << "Variable";
	}
	else if (element == "TrikWaitForMotion") {
	}
	else if (element == "TrikWaitForSonarDistance") {
		result  << "Distance" << "Port" << "Sign";
	}
	else if (element == "TrikWaitForTouchSensor") {
		result  << "Port";
	}
	else if (element == "TrikWaitGamepadButton") {
		result  << "Button";
	}
	else if (element == "TrikWaitGamepadConnect") {
	}
	else if (element == "TrikWaitGamepadDisconnect") {
	}
	else if (element == "TrikWaitGamepadWheel") {
		result  << "Angle" << "Sign";
	}
	else if (element == "TrikWaitPadPress") {
		result  << "Pad";
	}
	else if (element == "TrikWriteToFile") {
		result  << "File" << "Text";
	}
	else if (element == "VariableInit") {
		result  << "value" << "variable";
	}
	return result;
}

QStringList RobotsMetamodelPlugin::getPortTypes(const QString &/*diagram*/, const QString &element) const
{
	return mPortTypes[element];
}

QStringList RobotsMetamodelPlugin::getReferenceProperties(const QString &/*diagram*/, const QString &element) const
{
	QStringList result;
	if (element == "AbstractNode") {
	}
	else if (element == "ClearScreen") {
	}
	else if (element == "CommentBlock") {
	}
	else if (element == "ControlFlow") {
	}
	else if (element == "Ev3Beep") {
	}
	else if (element == "Ev3ClearEncoder") {
	}
	else if (element == "Ev3DrawCircle") {
	}
	else if (element == "Ev3DrawLine") {
	}
	else if (element == "Ev3DrawPixel") {
	}
	else if (element == "Ev3DrawRect") {
	}
	else if (element == "Ev3EnginesBackward") {
	}
	else if (element == "Ev3EnginesForward") {
	}
	else if (element == "Ev3EnginesStop") {
	}
	else if (element == "Ev3Led") {
	}
	else if (element == "Ev3PlayTone") {
	}
	else if (element == "Ev3WaitForButton") {
	}
	else if (element == "Ev3WaitForColor") {
	}
	else if (element == "Ev3WaitForColorIntensity") {
	}
	else if (element == "Ev3WaitForEncoder") {
	}
	else if (element == "Ev3WaitForLight") {
	}
	else if (element == "Ev3WaitForSonarDistance") {
	}
	else if (element == "Ev3WaitForSound") {
	}
	else if (element == "Ev3WaitForTouchSensor") {
	}
	else if (element == "FiBlock") {
	}
	else if (element == "FinalNode") {
	}
	else if (element == "Fork") {
	}
	else if (element == "Function") {
	}
	else if (element == "GetButtonCode") {
	}
	else if (element == "IfBlock") {
	}
	else if (element == "InitialNode") {
	}
	else if (element == "Join") {
	}
	else if (element == "KillThread") {
	}
	else if (element == "Loop") {
	}
	else if (element == "MarkerDown") {
	}
	else if (element == "MarkerUp") {
	}
	else if (element == "NxtBeep") {
	}
	else if (element == "NxtClearEncoder") {
	}
	else if (element == "NxtDrawCircle") {
	}
	else if (element == "NxtDrawLine") {
	}
	else if (element == "NxtDrawPixel") {
	}
	else if (element == "NxtDrawRect") {
	}
	else if (element == "NxtEnginesBackward") {
	}
	else if (element == "NxtEnginesForward") {
	}
	else if (element == "NxtEnginesStop") {
	}
	else if (element == "NxtPlayTone") {
	}
	else if (element == "NxtWaitForButton") {
	}
	else if (element == "NxtWaitForColor") {
	}
	else if (element == "NxtWaitForColorIntensity") {
	}
	else if (element == "NxtWaitForEncoder") {
	}
	else if (element == "NxtWaitForLight") {
	}
	else if (element == "NxtWaitForSonarDistance") {
	}
	else if (element == "NxtWaitForSound") {
	}
	else if (element == "NxtWaitForTouchSensor") {
	}
	else if (element == "PrintText") {
	}
	else if (element == "Randomizer") {
	}
	else if (element == "ReceiveMessageThreads") {
	}
	else if (element == "RobotsDiagramNode") {
	}
	else if (element == "SendMessageThreads") {
	}
	else if (element == "Subprogram") {
	}
	else if (element == "SubprogramDiagram") {
	}
	else if (element == "SwitchBlock") {
	}
	else if (element == "Timer") {
	}
	else if (element == "TrikAngularServo") {
	}
	else if (element == "TrikDetect") {
	}
	else if (element == "TrikDetectorToVariable") {
	}
	else if (element == "TrikDrawArc") {
	}
	else if (element == "TrikDrawEllipse") {
	}
	else if (element == "TrikDrawLine") {
	}
	else if (element == "TrikDrawPixel") {
	}
	else if (element == "TrikDrawRect") {
	}
	else if (element == "TrikInitCamera") {
	}
	else if (element == "TrikInitVideoStreaming") {
	}
	else if (element == "TrikLed") {
	}
	else if (element == "TrikPlayTone") {
	}
	else if (element == "TrikRemoveFile") {
	}
	else if (element == "TrikSadSmile") {
	}
	else if (element == "TrikSay") {
	}
	else if (element == "TrikSendMessage") {
	}
	else if (element == "TrikSetBackground") {
	}
	else if (element == "TrikSetPainterColor") {
	}
	else if (element == "TrikSetPainterWidth") {
	}
	else if (element == "TrikSmile") {
	}
	else if (element == "TrikSystem") {
	}
	else if (element == "TrikV62AngularServo") {
	}
	else if (element == "TrikV62ClearEncoder") {
	}
	else if (element == "TrikV62WaitForEncoder") {
	}
	else if (element == "TrikV6ClearEncoder") {
	}
	else if (element == "TrikV6EnginesBackward") {
	}
	else if (element == "TrikV6EnginesForward") {
	}
	else if (element == "TrikV6EnginesStop") {
	}
	else if (element == "TrikWaitForAccelerometer") {
	}
	else if (element == "TrikWaitForButton") {
	}
	else if (element == "TrikWaitForEncoder") {
	}
	else if (element == "TrikWaitForGyroscope") {
	}
	else if (element == "TrikWaitForIRDistance") {
	}
	else if (element == "TrikWaitForLight") {
	}
	else if (element == "TrikWaitForMessage") {
	}
	else if (element == "TrikWaitForMotion") {
	}
	else if (element == "TrikWaitForSonarDistance") {
	}
	else if (element == "TrikWaitForTouchSensor") {
	}
	else if (element == "TrikWaitGamepadButton") {
	}
	else if (element == "TrikWaitGamepadConnect") {
	}
	else if (element == "TrikWaitGamepadDisconnect") {
	}
	else if (element == "TrikWaitGamepadWheel") {
	}
	else if (element == "TrikWaitPadPress") {
	}
	else if (element == "TrikWriteToFile") {
	}
	else if (element == "VariableInit") {
	}
	return result;
}

QStringList RobotsMetamodelPlugin::getTypesContainedBy(const QString &element) const
{
	QStringList result;
	if (element == "RobotsDiagramNode") {
		result << "AbstractNode" ;
	}
	else if (element == "SubprogramDiagram") {
		result << "AbstractNode" ;
	}
	return result;
}

QList<QPair<QPair<QString,QString>,QPair<bool,QString> > > RobotsMetamodelPlugin::getPossibleEdges(const QString &element) const
{
	QList<QPair<QPair<QString,QString>,QPair<bool,QString> > > result;
	if (element == "ControlFlow") {
		result << qMakePair(qMakePair(QString("NonTyped"),QString("NonTyped")),qMakePair(false,QString("ControlFlow")));
	}
	return result;
}

//(-1) means "edge", (+1) means "node"
int RobotsMetamodelPlugin::isNodeOrEdge(const QString &element) const
{
	return mNodes.contains(element) ? 1 : (mEdges.contains(element) ? -1 : 0);
}
QString RobotsMetamodelPlugin::getGroupsXML() const 
{
	return QString::fromUtf8(" <groups>\n  <group name=\"RobotsDiagramGroup\" rootNode=\"diagram\">\n   <groupNode xPosition=\"0\" name=\"diagram\" yPosition=\"0\" type=\"RobotsDiagramNode\"/>\n   <groupNode xPosition=\"50\" name=\"initialNode\" parent=\"diagram\" yPosition=\"300\" type=\"InitialNode\"/>\n  </group>\n  <group name=\"SubprogramDiagramGroup\" rootNode=\"diagram\">\n   <groupNode xPosition=\"0\" name=\"diagram\" yPosition=\"0\" type=\"SubprogramDiagram\"/>\n   <groupNode xPosition=\"50\" name=\"initialNode\" parent=\"diagram\" yPosition=\"300\" type=\"InitialNode\"/>\n  </group>\n </groups>\n");
}

QList<QPair<QString, QString>> RobotsMetamodelPlugin::getEnumValues(const QString &name) const 
{
	if (name == "NxtButtons")
		return { qMakePair(QString("Enter"), tr("Enter")), qMakePair(QString("Escape"), tr("Escape")), qMakePair(QString("Left"), tr("Left")), qMakePair(QString("Right"), tr("Right")) };
	else if (name == "Color")
		return { qMakePair(QString("black"), tr("black")), qMakePair(QString("blue"), tr("blue")), qMakePair(QString("green"), tr("green")), qMakePair(QString("red"), tr("red")), qMakePair(QString("white"), tr("white")), qMakePair(QString("yellow"), tr("yellow")) };
	else if (name == "BreakEngineMode")
		return { qMakePair(QString("break"), tr("break")), qMakePair(QString("float"), tr("float")) };
	else if (name == "TrikV6AnalogSensorPort")
		return { qMakePair(QString("A1"), tr("A1")), qMakePair(QString("A2"), tr("A2")), qMakePair(QString("A3"), tr("A3")), qMakePair(QString("A4"), tr("A4")), qMakePair(QString("A5"), tr("A5")), qMakePair(QString("A6"), tr("A6")) };
	else if (name == "CallConcurrencyKind")
		return { qMakePair(QString("concurrent"), tr("Concurrent")), qMakePair(QString("guarded"), tr("Guarded")), qMakePair(QString("sequential"), tr("Sequential")) };
	else if (name == "VideoSensor")
		return { qMakePair(QString("color"), tr("Color sensor")), qMakePair(QString("line"), tr("Line sensor")), qMakePair(QString("object"), tr("Object sensor")) };
	else if (name == "DistanceSign")
		return { qMakePair(QString("greater"), tr("greater")), qMakePair(QString("less"), tr("less")), qMakePair(QString("notGreater"), tr("not greater")), qMakePair(QString("notLess"), tr("not less")) };
	else if (name == "Bool")
		return { qMakePair(QString("false"), tr("False")), qMakePair(QString("true"), tr("True")) };
	else if (name == "GuardType")
		return { qMakePair(QString("false"), tr("false")), qMakePair(QString("iteration"), tr("body")), qMakePair(QString("true"), tr("true")) };
	else if (name == "TrikV6DigitalSensorPort")
		return { qMakePair(QString("D1"), tr("D1")), qMakePair(QString("D2"), tr("D2")) };
	else if (name == "DetectableVideoSensor")
		return { qMakePair(QString("line"), tr("Line sensor")), qMakePair(QString("object"), tr("Object sensor")) };
	else if (name == "Ev3MotorPort")
		return { qMakePair(QString("A"), tr("A")), qMakePair(QString("B"), tr("B")), qMakePair(QString("C"), tr("C")), qMakePair(QString("D"), tr("D")) };
	else if (name == "Accelerations")
		return { qMakePair(QString("normal"), tr("norm")), qMakePair(QString("xAxis"), tr("x-axis")), qMakePair(QString("yAxis"), tr("y-axis")), qMakePair(QString("zAxis"), tr("z-axis")) };
	else if (name == "NxtSensorPort")
		return { qMakePair(QString("1"), tr("1")), qMakePair(QString("2"), tr("2")), qMakePair(QString("3"), tr("3")), qMakePair(QString("4"), tr("4")) };
	else if (name == "BackgroundColor")
		return { qMakePair(QString("black"), tr("black")), qMakePair(QString("blue"), tr("blue")), qMakePair(QString("cyan"), tr("cyan")), qMakePair(QString("darkBlue"), tr("dark blue")), qMakePair(QString("darkCyan"), tr("dark cyan")), qMakePair(QString("darkGray"), tr("dark gray")), qMakePair(QString("darkGreen"), tr("dark green")), qMakePair(QString("darkMagenta"), tr("dark magenta")), qMakePair(QString("darkRed"), tr("dark red")), qMakePair(QString("darkYellow"), tr("dark yellow")), qMakePair(QString("gray"), tr("gray")), qMakePair(QString("green"), tr("green")), qMakePair(QString("lightGray"), tr("light gray")), qMakePair(QString("magenta"), tr("magenta")), qMakePair(QString("red"), tr("red")), qMakePair(QString("white"), tr("white")), qMakePair(QString("yellow"), tr("yellow")) };
	else if (name == "AggregationKind")
		return { qMakePair(QString("composite"), tr("Composite")), qMakePair(QString("none"), tr("None")), qMakePair(QString("shared"), tr("Shared")) };
	else if (name == "TrikButtons")
		return { qMakePair(QString("Down"), tr("Down")), qMakePair(QString("Enter"), tr("Enter")), qMakePair(QString("Esc"), tr("Esc")), qMakePair(QString("Left"), tr("Left")), qMakePair(QString("Right"), tr("Right")), qMakePair(QString("Up"), tr("Up")) };
	else if (name == "Ev3LedColor")
		return { qMakePair(QString("green"), tr("green")), qMakePair(QString("green_flash"), tr("green flash")), qMakePair(QString("green_pulse"), tr("green pulse")), qMakePair(QString("off"), tr("off")), qMakePair(QString("orange"), tr("orange")), qMakePair(QString("orange_flash"), tr("orange flash")), qMakePair(QString("orange_pulse"), tr("orange pulse")), qMakePair(QString("red"), tr("red")), qMakePair(QString("red_flash"), tr("red flash")), qMakePair(QString("red_pulse"), tr("red pulse")) };
	else if (name == "TrikV6EncoderPort")
		return { qMakePair(QString("B1"), tr("B1")), qMakePair(QString("B2"), tr("B2")), qMakePair(QString("B3"), tr("B3")), qMakePair(QString("B4"), tr("B4")) };
	else if (name == "TrikV62EncoderPort")
		return { qMakePair(QString("E1"), tr("E1")), qMakePair(QString("E2"), tr("E2")), qMakePair(QString("E3"), tr("E3")), qMakePair(QString("E4"), tr("E4")) };
	else if (name == "Ev3SensorPort")
		return { qMakePair(QString("1"), tr("1")), qMakePair(QString("2"), tr("2")), qMakePair(QString("3"), tr("3")), qMakePair(QString("4"), tr("4")) };
	else if (name == "VisibilityKind")
		return { qMakePair(QString("package"), tr("Package")), qMakePair(QString("private"), tr("Private")), qMakePair(QString("protected"), tr("Protected")), qMakePair(QString("public"), tr("Public")) };
	else if (name == "ParameterDirectionKind")
		return { qMakePair(QString("in"), tr("In")), qMakePair(QString("inout"), tr("Inout")), qMakePair(QString("out"), tr("Out")), qMakePair(QString("return"), tr("Return")) };
	else if (name == "Ev3Buttons")
		return { qMakePair(QString("Back"), tr("Back")), qMakePair(QString("Down"), tr("Down")), qMakePair(QString("Enter"), tr("Enter")), qMakePair(QString("Left"), tr("Left")), qMakePair(QString("Right"), tr("Right")), qMakePair(QString("Up"), tr("Up")) };
	else if (name == "TrikLedColor")
		return { qMakePair(QString("green"), tr("green")), qMakePair(QString("off"), tr("off")), qMakePair(QString("orange"), tr("orange")), qMakePair(QString("red"), tr("red")) };
	return {};
}

bool RobotsMetamodelPlugin::isEnumEditable(const QString &name) const
{
	const QStringList editableEnums = { "GuardType" };
	return editableEnums.contains(name);
}

QString RobotsMetamodelPlugin::getPropertyType(const QString &element, const QString &property) const
{
	return mPropertyTypes[element][property];
}

QString RobotsMetamodelPlugin::getPropertyDefaultValue(const QString &element, const QString &property) const
{
	return mPropertyDefault[element][property];
}

QList<qReal::ListenerInterface*> RobotsMetamodelPlugin::listeners() const
{
	QList<qReal::ListenerInterface*> result;
	return result;
}
