#pragma once

#include <QtCore/QStringList>
#include <QtCore/QMap>
#include <QtGui/QIcon>
#include <QtCore/QPair>

#include "../../../../../qrgui/plugins/editorPluginInterface/editorInterface.h"

class RobotsMetamodelPlugin : public QObject, public qReal::EditorInterface
{
	Q_OBJECT
	Q_INTERFACES(qReal::EditorInterface)
	Q_PLUGIN_METADATA(IID "RobotsMetamodel")

public:

	RobotsMetamodelPlugin();

	QString id() const { return "RobotsMetamodel"; }
	QString version() const { return "3.1.3"; }

	QStringList diagrams() const override;
	QStringList elements(const QString &diagram) const override;
	QStringList getPropertiesWithDefaultValues(const QString &element) const override;

	QStringList getTypesContainedBy(const QString &element) const override;
	QList<QPair<QPair<QString, QString>, QPair<bool, QString>>> getPossibleEdges(QString const &element) const override;
	QList<qReal::EditorInterface::ExplosionData> explosions(const QString &diagram, QString const &element) const override;

	int isNodeOrEdge(const QString &element) const override;

	qReal::ElementImpl* getGraphicalObject(const QString &diagram, const QString &element) const override;
	QString getPropertyType(const QString &element, const QString &property) const override;
	QString getPropertyDefaultValue(const QString &element, const QString &property) const override;
	QStringList getPropertyNames(const QString &diagram, const QString &element) const override;
	QStringList getPortTypes(const QString &diagram, const QString &element) const override;
	QStringList getReferenceProperties(const QString &diagram, const QString &element) const override;
	QList<QPair<QString, QString>> getEnumValues(const QString &name) const override;
	bool isEnumEditable(const QString &name) const override;
	QString getGroupsXML() const override;
	QList<QPair<QString, QString>> getParentsOf(const QString &diagram, const QString &element) const override;

	QString editorName() const override;
	QString diagramName(const QString &diagram) const override;
	QString diagramNodeName(const QString &diagram) const override;
	QString elementName(const QString &diagram, const QString &element) const override;
	QString elementDescription(const QString &diagram, const QString &element) const override;
	QString propertyDescription(const QString &diagram, const QString &element, const QString &property) const override;
	QString propertyDisplayedName(const QString &diagram, const QString &element, const QString &property) const override;
	QString elementMouseGesture(const QString &digram, const QString &element) const override;

	QList<qReal::ListenerInterface*> listeners() const override;

	bool isParentOf(const QString &parentDiagram, const QString &parentElement, const QString &childDiagram, const QString &childElement) const override;

	QStringList diagramPaletteGroups(const QString &diagram) const override;
	QStringList diagramPaletteGroupList(const QString &diagram, const QString &group) const override;
	QString diagramPaletteGroupDescription(const QString &diagram, const QString &group) const override;
	bool shallPaletteBeSorted(const QString &diagram) const override;

private:
	virtual void initPlugin();
	virtual void initMouseGestureMap();
	virtual void initNameMap();
	virtual void initNodesAndEdgesSets();
	virtual void initPropertyMap();
	virtual void initPropertyDefaultsMap();
	virtual void initDescriptionMap();
	virtual void initPortTypes();
	virtual void initParentsMap();
	virtual void initPaletteGroupsMap();
	virtual void initPaletteGroupsDescriptionMap();
	virtual void initShallPaletteBeSortedMap();
	virtual void initExplosionsMap();

	QMap<QString, QIcon> mIconMap;
	QMap<QString, QString> mDiagramNameMap;
	QMap<QString, QString> mDiagramNodeNameMap;
	QSet<QString> mNodes;  // A set of nodes in this editor. Useful for deciding is element node or edge.
	QSet<QString> mEdges;  // A set of edges in this editor. Useful for deciding is element node or edge.
	QMap<QString, QMap<QString, QString>> mPropertyTypes;
	QMap<QString, QStringList> mPortTypes;
	QMap<QString, QMap<QString, QString>> mPropertyDefault;
	QMap<QString, QMap<QString, QString>> mElementsNameMap;
	QMap<QString, QMap<QString, QString>> mElementsDescriptionMap;
	QMap<QString, QMap<QString, QMap<QString, QString>>> mPropertiesDescriptionMap;
	QMap<QString, QMap<QString, QMap<QString, QString>>> mPropertiesDisplayedNamesMap;
	QMap<QString, QMap<QString, QString>> mElementMouseGesturesMap;
	QMap<QString, QMap<QString, QList<QPair<QString, QString>>>> mParentsMap;  // Maps diagram and element to a list of diagram-element pairs of parents (generalization relation).
	QMap<QString, QList<QPair<QString, QStringList>>> mPaletteGroupsMap;  // Maps element`s lists of all palette groups.
	QMap<QString, QMap<QString, QString>> mPaletteGroupsDescriptionMap;
	QMap<QString, bool> mShallPaletteBeSortedMap;
	QMap<QString, QMap<QString, QList<qReal::EditorInterface::ExplosionData>>> mExplosionsMap;
};

